@@ -94,6 +94,17 @@ public class CliFrontendOptions {
                                     + "program that was part of the program when the savepoint was triggered.")
                     .build();
 
+    public static final Option FLINK_CONFIG =
+            Option.builder("D")
+                    .required(false)
+                    .numberOfArgs(2)
+                    .valueSeparator('=')
+                    .argName("Session dynamic flink config key=val")
+                    .desc(
+                            "Allows specifying multiple flink generic configuration options. The available"
+                                    + "options can be found at https://nightlies.apache.org/flink/flink-docs-stable/ops/config.html")
+                    .build();
+
     public static Options initializeOptions() {
         return new Options()
                 .addOption(HELP)
@@ -105,6 +116,7 @@ public class CliFrontendOptions {
                 .addOption(USE_MINI_CLUSTER)
                 .addOption(SAVEPOINT_PATH_OPTION)
                 .addOption(SAVEPOINT_CLAIM_MODE)
-                .addOption(SAVEPOINT_ALLOW_NON_RESTORED_OPTION);
+                .addOption(SAVEPOINT_ALLOW_NON_RESTORED_OPTION)
+                .addOption(FLINK_CONFIG);
     }
 }
