@@ -30,4 +30,10 @@ public interface PipelineDefinitionParser {
      * the {@link PipelineDef}.
      */
     PipelineDef parse(Path pipelineDefPath, Configuration globalPipelineConfig) throws Exception;
+
+    /**
+     * Parse the specified pipeline definition string, merge global configurations, then generate
+     * the {@link PipelineDef}.
+     */
+    PipelineDef parse(String pipelineDefText, Configuration globalPipelineConfig) throws Exception;
 }
