@@ -21,7 +21,9 @@ import org.apache.flink.cdc.common.configuration.Configuration;
 import org.apache.flink.cdc.common.event.SchemaChangeEventType;
 import org.apache.flink.cdc.common.event.SchemaChangeEventTypeFamily;
 import org.apache.flink.cdc.common.pipeline.SchemaChangeBehavior;
+import org.apache.flink.cdc.common.utils.Preconditions;
 import org.apache.flink.cdc.common.utils.StringUtils;
+import org.apache.flink.cdc.composer.definition.ModelDef;
 import org.apache.flink.cdc.composer.definition.PipelineDef;
 import org.apache.flink.cdc.composer.definition.RouteDef;
 import org.apache.flink.cdc.composer.definition.SinkDef;
@@ -42,6 +44,7 @@ import java.util.List;
 import java.util.Map;
 import java.util.Optional;
 import java.util.Set;
+import java.util.stream.Stream;
 
 import static org.apache.flink.cdc.common.pipeline.PipelineOptions.PIPELINE_SCHEMA_CHANGE_BEHAVIOR;
 import static org.apache.flink.cdc.common.utils.ChangeEventUtils.resolveSchemaEvolutionOptions;
@@ -56,6 +59,7 @@ public class YamlPipelineDefinitionParser implements PipelineDefinitionParser {
     private static final String ROUTE_KEY = "route";
     private static final String TRANSFORM_KEY = "transform";
     private static final String PIPELINE_KEY = "pipeline";
+    private static final String MODEL_KEY = "model";
 
     // Source / sink keys
     private static final String TYPE_KEY = "type";
@@ -74,12 +78,19 @@ public class YamlPipelineDefinitionParser implements PipelineDefinitionParser {
     private static final String TRANSFORM_PROJECTION_KEY = "projection";
     private static final String TRANSFORM_FILTER_KEY = "filter";
     private static final String TRANSFORM_DESCRIPTION_KEY = "description";
+    private static final String TRANSFORM_CONVERTER_AFTER_TRANSFORM_KEY =
+            "converter-after-transform";
 
     // UDF related keys
     private static final String UDF_KEY = "user-defined-function";
     private static final String UDF_FUNCTION_NAME_KEY = "name";
     private static final String UDF_CLASSPATH_KEY = "classpath";
 
+    // Model related keys
+    private static final String MODEL_NAME_KEY = "model-name";
+
+    private static final String MODEL_CLASS_NAME_KEY = "class-name";
+
     public static final String TRANSFORM_PRIMARY_KEY_KEY = "primary-keys";
 
     public static final String TRANSFORM_PARTITION_KEY_KEY = "partition-keys";
@@ -107,8 +118,16 @@ public class YamlPipelineDefinitionParser implements PipelineDefinitionParser {
         // UDFs are optional. We parse UDF first and remove it from the pipelineDefJsonNode since
         // it's not of plain data types and must be removed before calling toPipelineConfig.
         List<UdfDef> udfDefs = new ArrayList<>();
-        Optional.ofNullable(((ObjectNode) pipelineDefJsonNode.get(PIPELINE_KEY)).remove(UDF_KEY))
-                .ifPresent(node -> node.forEach(udf -> udfDefs.add(toUdfDef(udf))));
+        final List<ModelDef> modelDefs = new ArrayList<>();
+        if (pipelineDefJsonNode.get(PIPELINE_KEY) != null) {
+            Optional.ofNullable(
+                            ((ObjectNode) pipelineDefJsonNode.get(PIPELINE_KEY)).remove(UDF_KEY))
+                    .ifPresent(node -> node.forEach(udf -> udfDefs.add(toUdfDef(udf))));
+
+            Optional.ofNullable(
+                            ((ObjectNode) pipelineDefJsonNode.get(PIPELINE_KEY)).remove(MODEL_KEY))
+                    .ifPresent(node -> modelDefs.addAll(parseModels(node)));
+        }
 
         // Pipeline configs are optional
         Configuration userPipelineConfig = toPipelineConfig(pipelineDefJsonNode.get(PIPELINE_KEY));
@@ -152,7 +171,7 @@ public class YamlPipelineDefinitionParser implements PipelineDefinitionParser {
         pipelineConfig.addAll(userPipelineConfig);
 
         return new PipelineDef(
-                sourceDef, sinkDef, routeDefs, transformDefs, udfDefs, pipelineConfig);
+                sourceDef, sinkDef, routeDefs, transformDefs, udfDefs, modelDefs, pipelineConfig);
     }
 
     private SourceDef toSourceDef(JsonNode sourceNode) {
@@ -175,6 +194,7 @@ public class YamlPipelineDefinitionParser implements PipelineDefinitionParser {
     private SinkDef toSinkDef(JsonNode sinkNode, SchemaChangeBehavior schemaChangeBehavior) {
         List<String> includedSETypes = new ArrayList<>();
         List<String> excludedSETypes = new ArrayList<>();
+        boolean excludedFieldNotPresent = sinkNode.get(EXCLUDE_SCHEMA_EVOLUTION_TYPES) == null;
 
         Optional.ofNullable(sinkNode.get(INCLUDE_SCHEMA_EVOLUTION_TYPES))
                 .ifPresent(e -> e.forEach(tag -> includedSETypes.add(tag.asText())));
@@ -190,6 +210,14 @@ public class YamlPipelineDefinitionParser implements PipelineDefinitionParser {
                     .forEach(includedSETypes::add);
         }
 
+        if (excludedFieldNotPresent && SchemaChangeBehavior.LENIENT.equals(schemaChangeBehavior)) {
+            // In lenient mode, we exclude DROP_TABLE and TRUNCATE_TABLE by default. This could be
+            // overridden by manually specifying excluded types.
+            Stream.of(SchemaChangeEventType.DROP_TABLE, SchemaChangeEventType.TRUNCATE_TABLE)
+                    .map(SchemaChangeEventType::getTag)
+                    .forEach(excludedSETypes::add);
+        }
+
         Set<SchemaChangeEventType> declaredSETypes =
                 resolveSchemaEvolutionOptions(includedSETypes, excludedSETypes);
 
@@ -290,6 +318,10 @@ public class YamlPipelineDefinitionParser implements PipelineDefinitionParser {
                 Optional.ofNullable(transformNode.get(TRANSFORM_DESCRIPTION_KEY))
                         .map(JsonNode::asText)
                         .orElse(null);
+        String postTransformConverter =
+                Optional.ofNullable(transformNode.get(TRANSFORM_CONVERTER_AFTER_TRANSFORM_KEY))
+                        .map(JsonNode::asText)
+                        .orElse(null);
 
         return new TransformDef(
                 sourceTable,
@@ -298,7 +330,8 @@ public class YamlPipelineDefinitionParser implements PipelineDefinitionParser {
                 primaryKeys,
                 partitionKeys,
                 tableOptions,
-                description);
+                description,
+                postTransformConverter);
     }
 
     private Configuration toPipelineConfig(JsonNode pipelineConfigNode) {
@@ -310,4 +343,34 @@ public class YamlPipelineDefinitionParser implements PipelineDefinitionParser {
                         pipelineConfigNode, new TypeReference<Map<String, String>>() {});
         return Configuration.fromMap(pipelineConfigMap);
     }
+
+    private List<ModelDef> parseModels(JsonNode modelsNode) {
+        List<ModelDef> modelDefs = new ArrayList<>();
+        Preconditions.checkNotNull(modelsNode, "`model` in `pipeline` should not be empty.");
+        if (modelsNode.isArray()) {
+            for (JsonNode modelNode : modelsNode) {
+                modelDefs.add(convertJsonNodeToModelDef(modelNode));
+            }
+        } else {
+            modelDefs.add(convertJsonNodeToModelDef(modelsNode));
+        }
+        return modelDefs;
+    }
+
+    private ModelDef convertJsonNodeToModelDef(JsonNode modelNode) {
+        String name =
+                checkNotNull(
+                                modelNode.get(MODEL_NAME_KEY),
+                                "Missing required field \"%s\" in `model`",
+                                MODEL_NAME_KEY)
+                        .asText();
+        String model =
+                checkNotNull(
+                                modelNode.get(MODEL_CLASS_NAME_KEY),
+                                "Missing required field \"%s\" in `model`",
+                                MODEL_CLASS_NAME_KEY)
+                        .asText();
+        Map<String, String> properties = mapper.convertValue(modelNode, Map.class);
+        return new ModelDef(name, model, properties);
+    }
 }
