@@ -22,12 +22,17 @@ import org.apache.flink.api.connector.sink2.Sink;
 import org.apache.flink.api.connector.sink2.TwoPhaseCommittingSink;
 import org.apache.flink.cdc.common.annotation.Internal;
 import org.apache.flink.cdc.common.annotation.VisibleForTesting;
+import org.apache.flink.cdc.common.configuration.Configuration;
 import org.apache.flink.cdc.common.event.Event;
+import org.apache.flink.cdc.common.factories.DataSinkFactory;
+import org.apache.flink.cdc.common.factories.FactoryHelper;
 import org.apache.flink.cdc.common.sink.DataSink;
 import org.apache.flink.cdc.common.sink.EventSinkProvider;
 import org.apache.flink.cdc.common.sink.FlinkSinkFunctionProvider;
 import org.apache.flink.cdc.common.sink.FlinkSinkProvider;
 import org.apache.flink.cdc.composer.definition.SinkDef;
+import org.apache.flink.cdc.composer.flink.FlinkEnvironmentUtils;
+import org.apache.flink.cdc.composer.utils.FactoryDiscoveryUtils;
 import org.apache.flink.cdc.runtime.operators.sink.DataSinkFunctionOperator;
 import org.apache.flink.cdc.runtime.operators.sink.DataSinkWriterOperatorFactory;
 import org.apache.flink.core.io.SimpleVersionedSerializer;
@@ -53,6 +58,25 @@ public class DataSinkTranslator {
     private static final String SINK_WRITER_PREFIX = "Sink Writer: ";
     private static final String SINK_COMMITTER_PREFIX = "Sink Committer: ";
 
+    public DataSink createDataSink(
+            SinkDef sinkDef, Configuration pipelineConfig, StreamExecutionEnvironment env) {
+        // Search the data sink factory
+        DataSinkFactory sinkFactory =
+                FactoryDiscoveryUtils.getFactoryByIdentifier(
+                        sinkDef.getType(), DataSinkFactory.class);
+
+        // Include sink connector JAR
+        FactoryDiscoveryUtils.getJarPathByIdentifier(sinkFactory)
+                .ifPresent(jar -> FlinkEnvironmentUtils.addJar(env, jar));
+
+        // Create data sink
+        return sinkFactory.createDataSink(
+                new FactoryHelper.DefaultContext(
+                        sinkDef.getConfig(),
+                        pipelineConfig,
+                        Thread.currentThread().getContextClassLoader()));
+    }
+
     public void translate(
             SinkDef sinkDef,
             DataStream<Event> input,
@@ -82,7 +106,7 @@ public class DataSinkTranslator {
             String sinkName,
             OperatorID schemaOperatorID) {
         DataStream<Event> stream = input;
-        // Pre write topology
+        // Pre-write topology
         if (sink instanceof WithPreWriteTopology) {
             stream = ((WithPreWriteTopology<Event>) sink).addPreWriteTopology(stream);
         }
