@@ -17,8 +17,10 @@
 
 package org.apache.flink.cdc.composer.flink.translator;
 
-import org.apache.flink.api.java.tuple.Tuple2;
+import org.apache.flink.api.java.tuple.Tuple3;
 import org.apache.flink.cdc.common.event.Event;
+import org.apache.flink.cdc.common.source.SupportedMetadataColumn;
+import org.apache.flink.cdc.composer.definition.ModelDef;
 import org.apache.flink.cdc.composer.definition.TransformDef;
 import org.apache.flink.cdc.composer.definition.UdfDef;
 import org.apache.flink.cdc.runtime.operators.transform.PostTransformOperator;
@@ -26,7 +28,9 @@ import org.apache.flink.cdc.runtime.operators.transform.PreTransformOperator;
 import org.apache.flink.cdc.runtime.typeutils.EventTypeInfo;
 import org.apache.flink.streaming.api.datastream.DataStream;
 
+import java.util.HashMap;
 import java.util.List;
+import java.util.Map;
 import java.util.stream.Collectors;
 
 /**
@@ -35,8 +39,17 @@ import java.util.stream.Collectors;
  */
 public class TransformTranslator {
 
+    /** Package of built-in model. */
+    public static final String PREFIX_CLASSPATH_BUILT_IN_MODEL =
+            "org.apache.flink.cdc.runtime.model.";
+
     public DataStream<Event> translatePreTransform(
-            DataStream<Event> input, List<TransformDef> transforms, List<UdfDef> udfFunctions) {
+            DataStream<Event> input,
+            List<TransformDef> transforms,
+            List<UdfDef> udfFunctions,
+            List<ModelDef> models,
+            SupportedMetadataColumn[] supportedMetadataColumns,
+            boolean canContainDistributedTables) {
         if (transforms.isEmpty()) {
             return input;
         }
@@ -46,16 +59,24 @@ public class TransformTranslator {
         for (TransformDef transform : transforms) {
             preTransformFunctionBuilder.addTransform(
                     transform.getSourceTable(),
-                    transform.getProjection().orElse(null),
-                    transform.getFilter().orElse(null),
+                    transform.getProjection(),
+                    transform.getFilter(),
                     transform.getPrimaryKeys(),
                     transform.getPartitionKeys(),
-                    transform.getTableOptions());
+                    transform.getTableOptions(),
+                    transform.getPostTransformConverter(),
+                    supportedMetadataColumns);
         }
-        preTransformFunctionBuilder.addUdfFunctions(
-                udfFunctions.stream()
-                        .map(udf -> Tuple2.of(udf.getName(), udf.getClasspath()))
-                        .collect(Collectors.toList()));
+
+        preTransformFunctionBuilder
+                .addUdfFunctions(
+                        udfFunctions.stream()
+                                .map(this::udfDefToUDFTuple)
+                                .collect(Collectors.toList()))
+                .addUdfFunctions(
+                        models.stream().map(this::modelToUDFTuple).collect(Collectors.toList()))
+                .canContainDistributedTables(canContainDistributedTables);
+
         return input.transform(
                 "Transform:Schema", new EventTypeInfo(), preTransformFunctionBuilder.build());
     }
@@ -64,7 +85,9 @@ public class TransformTranslator {
             DataStream<Event> input,
             List<TransformDef> transforms,
             String timezone,
-            List<UdfDef> udfFunctions) {
+            List<UdfDef> udfFunctions,
+            List<ModelDef> models,
+            SupportedMetadataColumn[] supportedMetadataColumns) {
         if (transforms.isEmpty()) {
             return input;
         }
@@ -75,19 +98,32 @@ public class TransformTranslator {
             if (transform.isValidProjection() || transform.isValidFilter()) {
                 postTransformFunctionBuilder.addTransform(
                         transform.getSourceTable(),
-                        transform.isValidProjection() ? transform.getProjection().get() : null,
-                        transform.isValidFilter() ? transform.getFilter().get() : null,
+                        transform.getProjection(),
+                        transform.getFilter(),
                         transform.getPrimaryKeys(),
                         transform.getPartitionKeys(),
-                        transform.getTableOptions());
+                        transform.getTableOptions(),
+                        transform.getPostTransformConverter(),
+                        supportedMetadataColumns);
             }
         }
         postTransformFunctionBuilder.addTimezone(timezone);
         postTransformFunctionBuilder.addUdfFunctions(
-                udfFunctions.stream()
-                        .map(udf -> Tuple2.of(udf.getName(), udf.getClasspath()))
-                        .collect(Collectors.toList()));
+                udfFunctions.stream().map(this::udfDefToUDFTuple).collect(Collectors.toList()));
+        postTransformFunctionBuilder.addUdfFunctions(
+                models.stream().map(this::modelToUDFTuple).collect(Collectors.toList()));
         return input.transform(
                 "Transform:Data", new EventTypeInfo(), postTransformFunctionBuilder.build());
     }
+
+    private Tuple3<String, String, Map<String, String>> modelToUDFTuple(ModelDef model) {
+        return Tuple3.of(
+                model.getModelName(),
+                PREFIX_CLASSPATH_BUILT_IN_MODEL + model.getClassName(),
+                model.getParameters());
+    }
+
+    private Tuple3<String, String, Map<String, String>> udfDefToUDFTuple(UdfDef udf) {
+        return Tuple3.of(udf.getName(), udf.getClasspath(), new HashMap<>());
+    }
 }
