@@ -36,6 +36,7 @@ import org.apache.flink.connector.base.source.reader.RecordEmitter;
 import io.debezium.connector.mysql.antlr.MySqlAntlrDdlParser;
 import io.debezium.jdbc.JdbcConnection;
 import io.debezium.relational.Column;
+import io.debezium.relational.RelationalDatabaseConnectorConfig;
 import io.debezium.relational.Table;
 import io.debezium.relational.TableId;
 import io.debezium.relational.Tables;
@@ -201,7 +202,8 @@ public class MySqlPipelineRecordEmitter extends MySqlRecordEmitter<Event> {
             Column column = columns.get(i);
 
             String colName = column.name();
-            DataType dataType = MySqlTypeUtils.fromDbzColumn(column);
+            DataType dataType =
+                    MySqlTypeUtils.fromDbzColumn(column, sourceConfig.isTreatTinyInt1AsBoolean());
             if (!column.isOptional()) {
                 dataType = dataType.notNull();
             }
@@ -211,6 +213,7 @@ public class MySqlPipelineRecordEmitter extends MySqlRecordEmitter<Event> {
                     column.comment(),
                     column.defaultValueExpression().orElse(null));
         }
+        tableBuilder.comment(table.comment());
 
         List<String> primaryKey = table.primaryKeyColumnNames();
         if (Objects.nonNull(primaryKey) && !primaryKey.isEmpty()) {
@@ -229,7 +232,16 @@ public class MySqlPipelineRecordEmitter extends MySqlRecordEmitter<Event> {
 
     private synchronized MySqlAntlrDdlParser getParser() {
         if (mySqlAntlrDdlParser == null) {
-            mySqlAntlrDdlParser = new MySqlAntlrDdlParser();
+            boolean includeComments =
+                    sourceConfig
+                            .getDbzConfiguration()
+                            .getBoolean(
+                                    RelationalDatabaseConnectorConfig.INCLUDE_SCHEMA_COMMENTS
+                                            .name(),
+                                    false);
+            mySqlAntlrDdlParser =
+                    new MySqlAntlrDdlParser(
+                            true, false, includeComments, null, Tables.TableFilter.includeAll());
         }
         return mySqlAntlrDdlParser;
     }
@@ -237,7 +249,9 @@ public class MySqlPipelineRecordEmitter extends MySqlRecordEmitter<Event> {
     private List<CreateTableEvent> generateCreateTableEvent(MySqlSourceConfig sourceConfig) {
         try (JdbcConnection jdbc = openJdbcConnection(sourceConfig)) {
             List<CreateTableEvent> createTableEventCache = new ArrayList<>();
-            List<TableId> capturedTableIds = listTables(jdbc, sourceConfig.getTableFilters());
+            List<TableId> capturedTableIds =
+                    listTables(
+                            jdbc, sourceConfig.getDatabaseFilter(), sourceConfig.getTableFilter());
             for (TableId tableId : capturedTableIds) {
                 Schema schema = getSchema(jdbc, tableId);
                 createTableEventCache.add(
