@@ -23,6 +23,7 @@ import org.apache.flink.cdc.common.event.DataChangeEvent;
 import org.apache.flink.cdc.common.event.Event;
 import org.apache.flink.cdc.common.event.SchemaChangeEvent;
 import org.apache.flink.cdc.common.event.TableId;
+import org.apache.flink.cdc.common.schema.Schema;
 import org.apache.flink.cdc.common.utils.SchemaUtils;
 import org.apache.flink.cdc.connectors.paimon.sink.v2.bucket.BucketWrapperChangeEvent;
 
@@ -66,13 +67,14 @@ public class PaimonRecordEventSerializer implements PaimonRecordSerializer<Event
                         new TableSchemaInfo(createTableEvent.getSchema(), zoneId));
             } else {
                 SchemaChangeEvent schemaChangeEvent = (SchemaChangeEvent) event;
-                schemaMaps.put(
-                        schemaChangeEvent.tableId(),
-                        new TableSchemaInfo(
-                                SchemaUtils.applySchemaChangeEvent(
-                                        schemaMaps.get(schemaChangeEvent.tableId()).getSchema(),
-                                        schemaChangeEvent),
-                                zoneId));
+                Schema schema = schemaMaps.get(schemaChangeEvent.tableId()).getSchema();
+                if (!SchemaUtils.isSchemaChangeEventRedundant(schema, schemaChangeEvent)) {
+                    schemaMaps.put(
+                            schemaChangeEvent.tableId(),
+                            new TableSchemaInfo(
+                                    SchemaUtils.applySchemaChangeEvent(schema, schemaChangeEvent),
+                                    zoneId));
+                }
             }
             return new PaimonEvent(tableId, null, true);
         } else if (event instanceof DataChangeEvent) {
