@@ -18,28 +18,37 @@
 package org.apache.flink.cdc.connectors.paimon.sink.v2;
 
 import org.apache.flink.api.common.JobID;
+import org.apache.flink.api.common.JobInfo;
+import org.apache.flink.api.common.TaskInfo;
 import org.apache.flink.api.common.operators.MailboxExecutor;
 import org.apache.flink.api.common.operators.ProcessingTimeService;
 import org.apache.flink.api.common.serialization.SerializationSchema;
 import org.apache.flink.api.common.typeutils.TypeSerializer;
 import org.apache.flink.api.connector.sink2.Committer;
 import org.apache.flink.api.connector.sink2.Sink;
+import org.apache.flink.api.java.tuple.Tuple2;
+import org.apache.flink.cdc.common.data.binary.BinaryRecordData;
 import org.apache.flink.cdc.common.data.binary.BinaryStringData;
 import org.apache.flink.cdc.common.event.AddColumnEvent;
 import org.apache.flink.cdc.common.event.CreateTableEvent;
 import org.apache.flink.cdc.common.event.DataChangeEvent;
 import org.apache.flink.cdc.common.event.DropColumnEvent;
+import org.apache.flink.cdc.common.event.DropTableEvent;
 import org.apache.flink.cdc.common.event.Event;
 import org.apache.flink.cdc.common.event.TableId;
+import org.apache.flink.cdc.common.event.TruncateTableEvent;
 import org.apache.flink.cdc.common.exceptions.SchemaEvolveException;
+import org.apache.flink.cdc.common.exceptions.UnsupportedSchemaChangeEventException;
 import org.apache.flink.cdc.common.schema.Column;
 import org.apache.flink.cdc.common.schema.Schema;
-import org.apache.flink.cdc.common.types.DataTypes;
+import org.apache.flink.cdc.common.types.DataType;
 import org.apache.flink.cdc.common.types.RowType;
 import org.apache.flink.cdc.connectors.paimon.sink.PaimonMetadataApplier;
 import org.apache.flink.cdc.runtime.typeutils.BinaryRecordDataGenerator;
 import org.apache.flink.metrics.MetricGroup;
 import org.apache.flink.metrics.groups.SinkWriterMetricGroup;
+import org.apache.flink.metrics.groups.UnregisteredMetricsGroup;
+import org.apache.flink.runtime.metrics.groups.InternalSinkCommitterMetricGroup;
 import org.apache.flink.streaming.runtime.operators.sink.committables.CommitRequestImpl;
 import org.apache.flink.table.api.EnvironmentSettings;
 import org.apache.flink.table.api.TableEnvironment;
@@ -47,14 +56,15 @@ import org.apache.flink.types.Row;
 import org.apache.flink.types.RowKind;
 import org.apache.flink.util.UserCodeClassLoader;
 
+import org.apache.calcite.sql.validate.SqlValidatorException;
 import org.apache.paimon.catalog.Catalog;
 import org.apache.paimon.flink.FlinkCatalogFactory;
 import org.apache.paimon.flink.sink.MultiTableCommittable;
 import org.apache.paimon.options.Options;
-import org.junit.jupiter.api.Assertions;
+import org.assertj.core.api.Assertions;
 import org.junit.jupiter.api.io.TempDir;
 import org.junit.jupiter.params.ParameterizedTest;
-import org.junit.jupiter.params.provider.ValueSource;
+import org.junit.jupiter.params.provider.CsvSource;
 
 import java.io.File;
 import java.io.IOException;
@@ -71,6 +81,8 @@ import java.util.Set;
 import java.util.UUID;
 import java.util.stream.Collectors;
 
+import static org.apache.flink.cdc.common.types.DataTypes.STRING;
+
 /** An ITCase for {@link PaimonWriter} and {@link PaimonCommitter}. */
 public class PaimonSinkITCase {
 
@@ -82,9 +94,8 @@ public class PaimonSinkITCase {
 
     private String warehouse;
 
-    private TableId table1;
-
-    private BinaryRecordDataGenerator generator;
+    private final TableId table1 = TableId.tableId("test", "table1");
+    private final TableId table2 = TableId.tableId("test", "table2");
 
     private static int checkpointId = 1;
 
@@ -111,7 +122,6 @@ public class PaimonSinkITCase {
         catalogOptions.setString("metastore", metastore);
         catalogOptions.setString("warehouse", warehouse);
         catalogOptions.setString("cache-enabled", "false");
-        table1 = TableId.tableId("test", "table1");
         if ("hive".equals(metastore)) {
             catalogOptions.setString("hadoop-conf-dir", HADOOP_CONF_DIR);
             catalogOptions.setString("hive-conf-dir", HIVE_CONF_DIR);
@@ -123,7 +133,7 @@ public class PaimonSinkITCase {
                                     + "'metastore'='hive', "
                                     + "'hadoop-conf-dir'='%s', "
                                     + "'hive-conf-dir'='%s', "
-                                    + "'cache-enabled'='false' "
+                                    + "'cache-enabled'='false'"
                                     + ")",
                             warehouse, HADOOP_CONF_DIR, HIVE_CONF_DIR));
         } else {
@@ -136,48 +146,35 @@ public class PaimonSinkITCase {
                 .dropDatabase(TEST_DATABASE, true, true);
     }
 
-    private List<Event> createTestEvents() throws SchemaEvolveException {
+    private List<Event> createTestEvents(boolean enableDeleteVectors) throws SchemaEvolveException {
         List<Event> testEvents = new ArrayList<>();
         // create table
         Schema schema =
                 Schema.newBuilder()
-                        .physicalColumn("col1", DataTypes.STRING())
-                        .physicalColumn("col2", DataTypes.STRING())
+                        .physicalColumn("col1", STRING())
+                        .physicalColumn("col2", STRING())
                         .primaryKey("col1")
                         .option("bucket", "1")
+                        .option("deletion-vectors.enabled", String.valueOf(enableDeleteVectors))
                         .build();
         CreateTableEvent createTableEvent = new CreateTableEvent(table1, schema);
         testEvents.add(createTableEvent);
         PaimonMetadataApplier metadataApplier = new PaimonMetadataApplier(catalogOptions);
         metadataApplier.applySchemaChange(createTableEvent);
 
-        generator =
-                new BinaryRecordDataGenerator(RowType.of(DataTypes.STRING(), DataTypes.STRING()));
         // insert
-        DataChangeEvent insertEvent1 =
-                DataChangeEvent.insertEvent(
-                        table1,
-                        generator.generate(
-                                new Object[] {
-                                    BinaryStringData.fromString("1"),
-                                    BinaryStringData.fromString("1")
-                                }));
-        testEvents.add(insertEvent1);
-        DataChangeEvent insertEvent2 =
-                DataChangeEvent.insertEvent(
-                        table1,
-                        generator.generate(
-                                new Object[] {
-                                    BinaryStringData.fromString("2"),
-                                    BinaryStringData.fromString("2")
-                                }));
-        testEvents.add(insertEvent2);
+        testEvents.add(
+                generateInsert(
+                        table1, Arrays.asList(Tuple2.of(STRING(), "1"), Tuple2.of(STRING(), "1"))));
+        testEvents.add(
+                generateInsert(
+                        table1, Arrays.asList(Tuple2.of(STRING(), "2"), Tuple2.of(STRING(), "2"))));
         return testEvents;
     }
 
     @ParameterizedTest
-    @ValueSource(strings = {"filesystem", "hive"})
-    public void testSinkWithDataChange(String metastore)
+    @CsvSource({"filesystem, true", "filesystem, false", "hive, true", "hive, false"})
+    public void testSinkWithDataChange(String metastore, boolean enableDeleteVector)
             throws IOException, InterruptedException, Catalog.DatabaseNotEmptyException,
                     Catalog.DatabaseNotExistException, SchemaEvolveException {
         initialize(metastore);
@@ -188,243 +185,158 @@ public class PaimonSinkITCase {
         Committer<MultiTableCommittable> committer = paimonSink.createCommitter();
 
         // insert
-        for (Event event : createTestEvents()) {
-            writer.write(event, null);
-        }
-        writer.flush(false);
-        Collection<Committer.CommitRequest<MultiTableCommittable>> commitRequests =
-                writer.prepareCommit().stream()
-                        .map(this::correctCheckpointId)
-                        .map(MockCommitRequestImpl::new)
-                        .collect(Collectors.toList());
-        committer.commit(commitRequests);
-        List<Row> result = new ArrayList<>();
-        tEnv.sqlQuery("select * from paimon_catalog.test.table1")
-                .execute()
-                .collect()
-                .forEachRemaining(result::add);
-        Assertions.assertEquals(
-                Arrays.asList(
-                        Row.ofKind(RowKind.INSERT, "1", "1"), Row.ofKind(RowKind.INSERT, "2", "2")),
-                result);
+        writeAndCommit(
+                writer, committer, createTestEvents(enableDeleteVector).toArray(new Event[0]));
+        Assertions.assertThat(fetchResults(table1))
+                .containsExactlyInAnyOrder(
+                        Row.ofKind(RowKind.INSERT, "1", "1"), Row.ofKind(RowKind.INSERT, "2", "2"));
 
         // delete
-        Event event =
-                DataChangeEvent.deleteEvent(
-                        TableId.tableId("test", "table1"),
-                        generator.generate(
-                                new Object[] {
-                                    BinaryStringData.fromString("1"),
-                                    BinaryStringData.fromString("1")
-                                }));
-        writer.write(event, null);
-        writer.flush(false);
-        commitRequests =
-                writer.prepareCommit().stream()
-                        .map(this::correctCheckpointId)
-                        .map(MockCommitRequestImpl::new)
-                        .collect(Collectors.toList());
-        committer.commit(commitRequests);
-        result = new ArrayList<>();
-        tEnv.sqlQuery("select * from paimon_catalog.test.table1")
-                .execute()
-                .collect()
-                .forEachRemaining(result::add);
-        Assertions.assertEquals(
-                Collections.singletonList(Row.ofKind(RowKind.INSERT, "2", "2")), result);
+        writeAndCommit(
+                writer,
+                committer,
+                generateDelete(
+                        table1, Arrays.asList(Tuple2.of(STRING(), "1"), Tuple2.of(STRING(), "1"))));
+
+        Assertions.assertThat(fetchResults(table1))
+                .containsExactlyInAnyOrder(Row.ofKind(RowKind.INSERT, "2", "2"));
 
         // update
-        event =
-                DataChangeEvent.updateEvent(
-                        TableId.tableId("test", "table1"),
-                        generator.generate(
-                                new Object[] {
-                                    BinaryStringData.fromString("2"),
-                                    BinaryStringData.fromString("2")
-                                }),
-                        generator.generate(
-                                new Object[] {
-                                    BinaryStringData.fromString("2"),
-                                    BinaryStringData.fromString("x")
-                                }));
-        writer.write(event, null);
-        writer.flush(false);
-        commitRequests =
-                writer.prepareCommit().stream()
-                        .map(this::correctCheckpointId)
-                        .map(MockCommitRequestImpl::new)
-                        .collect(Collectors.toList());
-        committer.commit(commitRequests);
-        result = new ArrayList<>();
-        tEnv.sqlQuery("select * from paimon_catalog.test.table1")
-                .execute()
-                .collect()
-                .forEachRemaining(result::add);
-        Assertions.assertEquals(
-                Collections.singletonList(Row.ofKind(RowKind.INSERT, "2", "x")), result);
+        writeAndCommit(
+                writer,
+                committer,
+                generateUpdate(
+                        table1,
+                        Arrays.asList(Tuple2.of(STRING(), "2"), Tuple2.of(STRING(), "2")),
+                        Arrays.asList(Tuple2.of(STRING(), "2"), Tuple2.of(STRING(), "x"))));
+        Assertions.assertThat(fetchResults(table1))
+                .containsExactlyInAnyOrder(Row.ofKind(RowKind.INSERT, "2", "x"));
 
-        result = new ArrayList<>();
-        tEnv.sqlQuery("select max_sequence_number from paimon_catalog.test.`table1$files`")
-                .execute()
-                .collect()
-                .forEachRemaining(result::add);
-        // Each commit will generate one sequence number(equal to checkpointId).
-        Assertions.assertEquals(
-                Arrays.asList(
-                        Row.ofKind(RowKind.INSERT, 1L),
-                        Row.ofKind(RowKind.INSERT, 2L),
-                        Row.ofKind(RowKind.INSERT, 3L)),
-                result);
+        if (enableDeleteVector) {
+            Assertions.assertThat(fetchMaxSequenceNumber(table1.getTableName()))
+                    .containsExactlyInAnyOrder(
+                            Row.ofKind(RowKind.INSERT, 1L), Row.ofKind(RowKind.INSERT, 3L));
+        } else {
+            Assertions.assertThat(fetchMaxSequenceNumber(table1.getTableName()))
+                    .containsExactlyInAnyOrder(
+                            Row.ofKind(RowKind.INSERT, 1L),
+                            Row.ofKind(RowKind.INSERT, 2L),
+                            Row.ofKind(RowKind.INSERT, 3L));
+        }
     }
 
     @ParameterizedTest
-    @ValueSource(strings = {"filesystem", "hive"})
-    public void testSinkWithSchemaChange(String metastore)
+    @CsvSource({"filesystem, true", "filesystem, false", "hive, true", "hive, false"})
+    public void testSinkWithSchemaChange(String metastore, boolean enableDeleteVector)
             throws IOException, InterruptedException, Catalog.DatabaseNotEmptyException,
                     Catalog.DatabaseNotExistException, SchemaEvolveException {
         initialize(metastore);
         PaimonSink<Event> paimonSink =
-                new PaimonSink(
+                new PaimonSink<>(
                         catalogOptions, new PaimonRecordEventSerializer(ZoneId.systemDefault()));
         PaimonWriter<Event> writer = paimonSink.createWriter(new MockInitContext());
         Committer<MultiTableCommittable> committer = paimonSink.createCommitter();
 
         // 1. receive only DataChangeEvents during one checkpoint
-        for (Event event : createTestEvents()) {
-            writer.write(event, null);
-        }
-        writer.flush(false);
-        Collection<Committer.CommitRequest<MultiTableCommittable>> commitRequests =
-                writer.prepareCommit().stream()
-                        .map(this::correctCheckpointId)
-                        .map(MockCommitRequestImpl::new)
-                        .collect(Collectors.toList());
-        committer.commit(commitRequests);
-        List<Row> result = new ArrayList<>();
-        tEnv.sqlQuery("select * from paimon_catalog.test.table1")
-                .execute()
-                .collect()
-                .forEachRemaining(result::add);
-        Assertions.assertEquals(
-                Arrays.asList(
-                        Row.ofKind(RowKind.INSERT, "1", "1"), Row.ofKind(RowKind.INSERT, "2", "2")),
-                result);
+        writeAndCommit(
+                writer, committer, createTestEvents(enableDeleteVector).toArray(new Event[0]));
+        Assertions.assertThat(fetchResults(table1))
+                .containsExactlyInAnyOrder(
+                        Row.ofKind(RowKind.INSERT, "1", "1"), Row.ofKind(RowKind.INSERT, "2", "2"));
 
         // 2. receive DataChangeEvents and SchemaChangeEvents during one checkpoint
-        DataChangeEvent insertEvent3 =
-                DataChangeEvent.insertEvent(
-                        table1,
-                        generator.generate(
-                                new Object[] {
-                                    BinaryStringData.fromString("3"),
-                                    BinaryStringData.fromString("3")
-                                }));
-        writer.write(insertEvent3, null);
-        writer.flush(false);
+        writeAndCommit(
+                writer,
+                committer,
+                generateInsert(
+                        table1, Arrays.asList(Tuple2.of(STRING(), "3"), Tuple2.of(STRING(), "3"))));
 
         // add column
         AddColumnEvent.ColumnWithPosition columnWithPosition =
-                new AddColumnEvent.ColumnWithPosition(
-                        Column.physicalColumn("col3", DataTypes.STRING()));
+                new AddColumnEvent.ColumnWithPosition(Column.physicalColumn("col3", STRING()));
         AddColumnEvent addColumnEvent =
                 new AddColumnEvent(table1, Collections.singletonList(columnWithPosition));
         PaimonMetadataApplier metadataApplier = new PaimonMetadataApplier(catalogOptions);
         metadataApplier.applySchemaChange(addColumnEvent);
         writer.write(addColumnEvent, null);
-        generator =
-                new BinaryRecordDataGenerator(
-                        RowType.of(DataTypes.STRING(), DataTypes.STRING(), DataTypes.STRING()));
-        DataChangeEvent insertEvent4 =
-                DataChangeEvent.insertEvent(
+
+        writeAndCommit(
+                writer,
+                committer,
+                generateInsert(
                         table1,
-                        generator.generate(
-                                new Object[] {
-                                    BinaryStringData.fromString("4"),
-                                    BinaryStringData.fromString("4"),
-                                    BinaryStringData.fromString("4")
-                                }));
-        writer.write(insertEvent4, null);
-        writer.flush(false);
-        commitRequests =
-                writer.prepareCommit().stream()
-                        .map(this::correctCheckpointId)
-                        .map(MockCommitRequestImpl::new)
-                        .collect(Collectors.toList());
-        committer.commit(commitRequests);
-        result = new ArrayList<>();
-        tEnv.sqlQuery("select * from paimon_catalog.test.table1")
-                .execute()
-                .collect()
-                .forEachRemaining(result::add);
-        Assertions.assertEquals(
-                Arrays.asList(
+                        Arrays.asList(
+                                Tuple2.of(STRING(), "4"),
+                                Tuple2.of(STRING(), "4"),
+                                Tuple2.of(STRING(), "4"))));
+
+        Assertions.assertThat(fetchResults(table1))
+                .containsExactlyInAnyOrder(
                         Row.ofKind(RowKind.INSERT, "1", "1", null),
                         Row.ofKind(RowKind.INSERT, "2", "2", null),
                         Row.ofKind(RowKind.INSERT, "3", "3", null),
-                        Row.ofKind(RowKind.INSERT, "4", "4", "4")),
-                result);
+                        Row.ofKind(RowKind.INSERT, "4", "4", "4"));
 
         // 2. receive DataChangeEvents and SchemaChangeEvents during one checkpoint
-        DataChangeEvent insertEvent5 =
-                DataChangeEvent.insertEvent(
+        writeAndCommit(
+                writer,
+                committer,
+                generateInsert(
                         table1,
-                        generator.generate(
-                                new Object[] {
-                                    BinaryStringData.fromString("5"),
-                                    BinaryStringData.fromString("5"),
-                                    BinaryStringData.fromString("5")
-                                }));
-        writer.write(insertEvent5, null);
-        writer.flush(false);
+                        Arrays.asList(
+                                Tuple2.of(STRING(), "5"),
+                                Tuple2.of(STRING(), "5"),
+                                Tuple2.of(STRING(), "5"))));
+
         // drop column
         DropColumnEvent dropColumnEvent =
                 new DropColumnEvent(table1, Collections.singletonList("col2"));
         metadataApplier.applySchemaChange(dropColumnEvent);
         writer.write(dropColumnEvent, null);
-        generator =
-                new BinaryRecordDataGenerator(RowType.of(DataTypes.STRING(), DataTypes.STRING()));
-        DataChangeEvent insertEvent6 =
-                DataChangeEvent.insertEvent(
-                        table1,
-                        generator.generate(
-                                new Object[] {
-                                    BinaryStringData.fromString("6"),
-                                    BinaryStringData.fromString("6")
-                                }));
-        writer.write(insertEvent6, null);
-        writer.flush(false);
-        commitRequests =
-                writer.prepareCommit().stream()
-                        .map(this::correctCheckpointId)
-                        .map(MockCommitRequestImpl::new)
-                        .collect(Collectors.toList());
-        committer.commit(commitRequests);
-        result = new ArrayList<>();
-        tEnv.sqlQuery("select * from paimon_catalog.test.table1")
-                .execute()
-                .collect()
-                .forEachRemaining(result::add);
-        Assertions.assertEquals(
-                Arrays.asList(
+
+        writeAndCommit(
+                writer,
+                committer,
+                generateInsert(
+                        table1, Arrays.asList(Tuple2.of(STRING(), "6"), Tuple2.of(STRING(), "6"))));
+
+        List<Row> result = fetchResults(TableId.tableId("test", "`table1$files`"));
+        Set<Row> deduplicated = new HashSet<>(result);
+        Assertions.assertThat(result).hasSameSizeAs(deduplicated);
+
+        Assertions.assertThat(fetchResults(table1))
+                .containsExactlyInAnyOrder(
                         Row.ofKind(RowKind.INSERT, "1", null),
                         Row.ofKind(RowKind.INSERT, "2", null),
                         Row.ofKind(RowKind.INSERT, "3", null),
                         Row.ofKind(RowKind.INSERT, "4", "4"),
                         Row.ofKind(RowKind.INSERT, "5", "5"),
-                        Row.ofKind(RowKind.INSERT, "6", "6")),
-                result);
-        result = new ArrayList<>();
-        tEnv.sqlQuery("select min_sequence_number from paimon_catalog.test.`table1$files`")
-                .execute()
-                .collect()
-                .forEachRemaining(result::add);
-        Set<Row> deduplicated = new HashSet<>(result);
-        Assertions.assertEquals(result.size(), deduplicated.size());
+                        Row.ofKind(RowKind.INSERT, "6", "6"));
+
+        TruncateTableEvent truncateTableEvent = new TruncateTableEvent(table1);
+        if (enableDeleteVector) {
+            Assertions.assertThatThrownBy(
+                            () -> metadataApplier.applySchemaChange(truncateTableEvent))
+                    .isExactlyInstanceOf(SchemaEvolveException.class)
+                    .cause()
+                    .isExactlyInstanceOf(UnsupportedSchemaChangeEventException.class)
+                    .extracting("exceptionMessage")
+                    .isEqualTo("Unable to truncate a table with deletion vectors enabled.");
+        } else {
+            metadataApplier.applySchemaChange(truncateTableEvent);
+            Assertions.assertThat(fetchResults(table1)).isEmpty();
+        }
+
+        DropTableEvent dropTableEvent = new DropTableEvent(table1);
+        metadataApplier.applySchemaChange(dropTableEvent);
+        Assertions.assertThatThrownBy(() -> fetchResults(table1))
+                .hasRootCauseExactlyInstanceOf(SqlValidatorException.class)
+                .hasRootCauseMessage("Object 'table1' not found within 'paimon_catalog.test'");
     }
 
     @ParameterizedTest
-    @ValueSource(strings = {"filesystem", "hive"})
-    public void testSinkWithMultiTables(String metastore)
+    @CsvSource({"filesystem, true", "filesystem, false", "hive, true", "hive, false"})
+    public void testSinkWithMultiTables(String metastore, boolean enableDeleteVector)
             throws IOException, InterruptedException, Catalog.DatabaseNotEmptyException,
                     Catalog.DatabaseNotExistException, SchemaEvolveException {
         initialize(metastore);
@@ -433,13 +345,12 @@ public class PaimonSinkITCase {
                         catalogOptions, new PaimonRecordEventSerializer(ZoneId.systemDefault()));
         PaimonWriter<Event> writer = paimonSink.createWriter(new MockInitContext());
         Committer<MultiTableCommittable> committer = paimonSink.createCommitter();
-        List<Event> testEvents = createTestEvents();
+        List<Event> testEvents = createTestEvents(enableDeleteVector);
         // create table
-        TableId table2 = TableId.tableId("test", "table2");
         Schema schema =
                 Schema.newBuilder()
-                        .physicalColumn("col1", DataTypes.STRING())
-                        .physicalColumn("col2", DataTypes.STRING())
+                        .physicalColumn("col1", STRING())
+                        .physicalColumn("col2", STRING())
                         .primaryKey("col1")
                         .option("bucket", "1")
                         .build();
@@ -448,48 +359,94 @@ public class PaimonSinkITCase {
         PaimonMetadataApplier metadataApplier = new PaimonMetadataApplier(catalogOptions);
         metadataApplier.applySchemaChange(createTableEvent);
         // insert
-        DataChangeEvent insertEvent1 =
-                DataChangeEvent.insertEvent(
-                        table2,
-                        generator.generate(
-                                new Object[] {
-                                    BinaryStringData.fromString("1"),
-                                    BinaryStringData.fromString("1")
-                                }));
-        testEvents.add(insertEvent1);
+        testEvents.add(
+                generateInsert(
+                        table2, Arrays.asList(Tuple2.of(STRING(), "1"), Tuple2.of(STRING(), "1"))));
 
         // insert
-        for (Event event : testEvents) {
-            writer.write(event, null);
-        }
-        writer.flush(false);
+        writeAndCommit(writer, committer, testEvents.toArray(new Event[0]));
+
+        Assertions.assertThat(fetchResults(table1))
+                .containsExactlyInAnyOrder(
+                        Row.ofKind(RowKind.INSERT, "1", "1"), Row.ofKind(RowKind.INSERT, "2", "2"));
+        Assertions.assertThat(fetchResults(table2))
+                .containsExactlyInAnyOrder(Row.ofKind(RowKind.INSERT, "1", "1"));
+    }
+
+    private static void commit(
+            PaimonWriter<Event> writer, Committer<MultiTableCommittable> committer)
+            throws IOException, InterruptedException {
         Collection<Committer.CommitRequest<MultiTableCommittable>> commitRequests =
                 writer.prepareCommit().stream()
-                        .map(this::correctCheckpointId)
+                        .map(PaimonSinkITCase::correctCheckpointId)
                         .map(MockCommitRequestImpl::new)
                         .collect(Collectors.toList());
         committer.commit(commitRequests);
-        List<Row> result = new ArrayList<>();
-        tEnv.sqlQuery("select * from paimon_catalog.test.table1")
+    }
+
+    private static void writeAndCommit(
+            PaimonWriter<Event> writer, Committer<MultiTableCommittable> committer, Event... events)
+            throws IOException, InterruptedException {
+        for (Event event : events) {
+            writer.write(event, null);
+        }
+        writer.flush(false);
+        commit(writer, committer);
+    }
+
+    private List<Row> fetchResults(TableId tableId) {
+        List<Row> results = new ArrayList<>();
+        tEnv.sqlQuery("select * from paimon_catalog." + tableId.toString())
                 .execute()
                 .collect()
-                .forEachRemaining(result::add);
-        Assertions.assertEquals(
-                Arrays.asList(
-                        Row.ofKind(RowKind.INSERT, "1", "1"), Row.ofKind(RowKind.INSERT, "2", "2")),
-                result);
-        result = new ArrayList<>();
-        tEnv.sqlQuery("select * from paimon_catalog.test.table2")
+                .forEachRemaining(results::add);
+        return results;
+    }
+
+    private List<Row> fetchMaxSequenceNumber(String tableName) {
+        List<Row> results = new ArrayList<>();
+        tEnv.sqlQuery(
+                        "select max_sequence_number from paimon_catalog.test.`"
+                                + tableName
+                                + "$files`")
                 .execute()
                 .collect()
-                .forEachRemaining(result::add);
-        Assertions.assertEquals(
-                Collections.singletonList(Row.ofKind(RowKind.INSERT, "1", "1")), result);
+                .forEachRemaining(results::add);
+        return results;
+    }
+
+    private BinaryRecordData generate(List<Tuple2<DataType, Object>> elements) {
+        BinaryRecordDataGenerator generator =
+                new BinaryRecordDataGenerator(
+                        RowType.of(elements.stream().map(e -> e.f0).toArray(DataType[]::new)));
+        return generator.generate(
+                elements.stream()
+                        .map(e -> e.f1)
+                        .map(o -> o instanceof String ? BinaryStringData.fromString((String) o) : o)
+                        .toArray(Object[]::new));
+    }
+
+    private DataChangeEvent generateInsert(
+            TableId tableId, List<Tuple2<DataType, Object>> elements) {
+        return DataChangeEvent.insertEvent(tableId, generate(elements));
+    }
+
+    private DataChangeEvent generateUpdate(
+            TableId tableId,
+            List<Tuple2<DataType, Object>> beforeElements,
+            List<Tuple2<DataType, Object>> afterElements) {
+        return DataChangeEvent.updateEvent(
+                tableId, generate(beforeElements), generate(afterElements));
+    }
+
+    private DataChangeEvent generateDelete(
+            TableId tableId, List<Tuple2<DataType, Object>> elements) {
+        return DataChangeEvent.deleteEvent(tableId, generate(elements));
     }
 
     @ParameterizedTest
-    @ValueSource(strings = {"filesystem", "hive"})
-    public void testDuplicateCommitAfterRestore(String metastore)
+    @CsvSource({"filesystem, true", "filesystem, false", "hive, true", "hive, false"})
+    public void testDuplicateCommitAfterRestore(String metastore, boolean enableDeleteVector)
             throws IOException, InterruptedException, Catalog.DatabaseNotEmptyException,
                     Catalog.DatabaseNotExistException, SchemaEvolveException {
         initialize(metastore);
@@ -500,13 +457,13 @@ public class PaimonSinkITCase {
         Committer<MultiTableCommittable> committer = paimonSink.createCommitter();
 
         // insert
-        for (Event event : createTestEvents()) {
+        for (Event event : createTestEvents(enableDeleteVector)) {
             writer.write(event, null);
         }
         writer.flush(false);
         Collection<Committer.CommitRequest<MultiTableCommittable>> commitRequests =
                 writer.prepareCommit().stream()
-                        .map(this::correctCheckpointId)
+                        .map(PaimonSinkITCase::correctCheckpointId)
                         .map(MockCommitRequestImpl::new)
                         .collect(Collectors.toList());
         committer.commit(commitRequests);
@@ -521,25 +478,24 @@ public class PaimonSinkITCase {
             // CommitterOperator will try to re-commit recovered transactions.
             committer.commit(commitRequests);
             List<DataChangeEvent> events =
-                    Arrays.asList(
-                            DataChangeEvent.insertEvent(
+                    Collections.singletonList(
+                            generateInsert(
                                     table1,
-                                    generator.generate(
-                                            new Object[] {
-                                                BinaryStringData.fromString(Integer.toString(i)),
-                                                BinaryStringData.fromString(Integer.toString(i))
-                                            })));
-            Assertions.assertDoesNotThrow(
-                    () -> {
-                        for (Event event : events) {
-                            writer.write(event, null);
-                        }
-                    });
+                                    Arrays.asList(
+                                            Tuple2.of(STRING(), String.valueOf(i)),
+                                            Tuple2.of(STRING(), String.valueOf(i)))));
+            Assertions.assertThatCode(
+                            () -> {
+                                for (Event event : events) {
+                                    writer.write(event, null);
+                                }
+                            })
+                    .doesNotThrowAnyException();
             writer.flush(false);
             // Checkpoint id start from 1
             committer.commit(
                     writer.prepareCommit().stream()
-                            .map(this::correctCheckpointId)
+                            .map(PaimonSinkITCase::correctCheckpointId)
                             .map(MockCommitRequestImpl::new)
                             .collect(Collectors.toList()));
         }
@@ -549,16 +505,21 @@ public class PaimonSinkITCase {
                 .execute()
                 .collect()
                 .forEachRemaining(result::add);
-        // 8 APPEND and 1 COMPACT
-        Assertions.assertEquals(result.size(), 9);
+        if (enableDeleteVector) {
+            // Each APPEND will trigger COMPACT once enable deletion-vectors.
+            Assertions.assertThat(result).hasSize(16);
+        } else {
+            // 8 APPEND and 1 COMPACT
+            Assertions.assertThat(result).hasSize(9);
+        }
         result.clear();
 
         tEnv.sqlQuery("select * from paimon_catalog.test.`table1`")
                 .execute()
                 .collect()
                 .forEachRemaining(result::add);
-        Assertions.assertEquals(
-                Arrays.asList(
+        Assertions.assertThat(result)
+                .containsExactlyInAnyOrder(
                         Row.ofKind(RowKind.INSERT, "1", "1"),
                         Row.ofKind(RowKind.INSERT, "2", "2"),
                         Row.ofKind(RowKind.INSERT, "3", "3"),
@@ -566,11 +527,10 @@ public class PaimonSinkITCase {
                         Row.ofKind(RowKind.INSERT, "5", "5"),
                         Row.ofKind(RowKind.INSERT, "6", "6"),
                         Row.ofKind(RowKind.INSERT, "7", "7"),
-                        Row.ofKind(RowKind.INSERT, "8", "8")),
-                result);
+                        Row.ofKind(RowKind.INSERT, "8", "8"));
     }
 
-    private MultiTableCommittable correctCheckpointId(MultiTableCommittable committable) {
+    private static MultiTableCommittable correctCheckpointId(MultiTableCommittable committable) {
         // update the right checkpointId for MultiTableCommittable
         return new MultiTableCommittable(
                 committable.getDatabase(),
@@ -583,7 +543,10 @@ public class PaimonSinkITCase {
     private static class MockCommitRequestImpl<CommT> extends CommitRequestImpl<CommT> {
 
         protected MockCommitRequestImpl(CommT committable) {
-            super(committable);
+            super(
+                    committable,
+                    InternalSinkCommitterMetricGroup.wrap(
+                            UnregisteredMetricsGroup.createOperatorMetricGroup()));
         }
     }
 
@@ -644,5 +607,15 @@ public class PaimonSinkITCase {
         public JobID getJobId() {
             return null;
         }
+
+        @Override
+        public JobInfo getJobInfo() {
+            return null;
+        }
+
+        @Override
+        public TaskInfo getTaskInfo() {
+            return null;
+        }
     }
 }
