@@ -18,9 +18,11 @@
 package org.apache.flink.cdc.connectors.paimon.sink.v2;
 
 import org.apache.flink.cdc.common.data.DecimalData;
+import org.apache.flink.cdc.common.data.GenericMapData;
 import org.apache.flink.cdc.common.data.LocalZonedTimestampData;
 import org.apache.flink.cdc.common.data.RecordData;
 import org.apache.flink.cdc.common.data.TimestampData;
+import org.apache.flink.cdc.common.data.binary.BinaryMapData;
 import org.apache.flink.cdc.common.data.binary.BinaryRecordData;
 import org.apache.flink.cdc.common.data.binary.BinaryStringData;
 import org.apache.flink.cdc.common.event.DataChangeEvent;
@@ -28,11 +30,14 @@ import org.apache.flink.cdc.common.event.TableId;
 import org.apache.flink.cdc.common.schema.Schema;
 import org.apache.flink.cdc.common.types.DataTypes;
 import org.apache.flink.cdc.common.types.RowType;
+import org.apache.flink.cdc.runtime.serializer.data.MapDataSerializer;
 import org.apache.flink.cdc.runtime.typeutils.BinaryRecordDataGenerator;
 
 import org.apache.paimon.data.BinaryString;
 import org.apache.paimon.data.Decimal;
 import org.apache.paimon.data.GenericRow;
+import org.apache.paimon.data.InternalMap;
+import org.apache.paimon.data.NestedRow;
 import org.apache.paimon.data.Timestamp;
 import org.apache.paimon.types.RowKind;
 import org.junit.jupiter.api.Assertions;
@@ -41,7 +46,9 @@ import org.junit.jupiter.api.Test;
 import java.math.BigDecimal;
 import java.time.Instant;
 import java.time.ZoneId;
+import java.util.HashMap;
 import java.util.List;
+import java.util.Map;
 
 /** Tests for {@link PaimonWriterHelper}. */
 public class PaimonWriterHelperTest {
@@ -171,4 +178,78 @@ public class PaimonWriterHelperTest {
         genericRow = PaimonWriterHelper.convertEventToGenericRow(dataChangeEvent, fieldGetters);
         Assertions.assertEquals(genericRow.getRowKind(), RowKind.INSERT);
     }
+
+    @Test
+    public void testConvertEventToGenericRowWithNestedRow() {
+        // Define the inner row type with an integer and a map
+        RowType innerRowType =
+                RowType.of(DataTypes.INT(), DataTypes.MAP(DataTypes.STRING(), DataTypes.STRING()));
+
+        // Define the outer row type with a nested row
+        RowType rowType = RowType.of(DataTypes.ROW(innerRowType));
+
+        // Create a map serializer and a map data instance
+        MapDataSerializer mapDataserializer =
+                new MapDataSerializer(DataTypes.STRING(), DataTypes.STRING());
+        Map<BinaryStringData, BinaryStringData> map = new HashMap<>();
+        map.put(BinaryStringData.fromString("key1"), BinaryStringData.fromString("value1"));
+        map.put(BinaryStringData.fromString("key2"), BinaryStringData.fromString("value2"));
+        GenericMapData genMapData = new GenericMapData(map);
+        BinaryMapData binMap = mapDataserializer.toBinaryMap(genMapData);
+
+        // Create inner row data
+        Object[] innerRowData =
+                new Object[] {
+                    42, // Integer field
+                    binMap // Map field
+                };
+
+        // Generate inner BinaryRecordData
+        BinaryRecordData innerRecordData =
+                new BinaryRecordDataGenerator(innerRowType).generate(innerRowData);
+
+        // Create outer row data containing the inner record data
+        Object[] testData = new Object[] {innerRecordData};
+
+        // Generate outer BinaryRecordData
+        BinaryRecordData recordData = new BinaryRecordDataGenerator(rowType).generate(testData);
+
+        // Create schema and field getters
+        Schema schema = Schema.newBuilder().fromRowDataType(rowType).build();
+        List<RecordData.FieldGetter> fieldGetters =
+                PaimonWriterHelper.createFieldGetters(schema, ZoneId.of("UTC+8"));
+
+        // Create a data change event
+        DataChangeEvent dataChangeEvent =
+                DataChangeEvent.insertEvent(TableId.parse("database.table"), recordData);
+
+        // Convert the event to GenericRow
+        GenericRow genericRow =
+                PaimonWriterHelper.convertEventToGenericRow(dataChangeEvent, fieldGetters);
+
+        // Verify the nested row field
+        NestedRow innerRow = (NestedRow) genericRow.getField(0);
+        NestedRow nestedRow = (NestedRow) innerRow.getRow(0, 2);
+        int actual = nestedRow.getRow(0, 2).getInt(0); // 42
+        Assertions.assertEquals(42, actual);
+        Map<BinaryString, BinaryString> expectedMap = new HashMap<>();
+        expectedMap.put(BinaryString.fromString("key1"), BinaryString.fromString("value1"));
+        expectedMap.put(BinaryString.fromString("key2"), BinaryString.fromString("value2"));
+        InternalMap internalMap = nestedRow.getMap(1);
+        Map<BinaryString, BinaryString> extractedMap = extractMap(internalMap);
+
+        for (Map.Entry<BinaryString, BinaryString> entry : expectedMap.entrySet()) {
+            Assertions.assertEquals(entry.getValue(), extractedMap.get(entry.getKey()));
+        }
+    }
+
+    private static Map<BinaryString, BinaryString> extractMap(InternalMap internalMap) {
+        Map<BinaryString, BinaryString> resultMap = new HashMap<>();
+        for (int i = 0; i < internalMap.size(); i++) {
+            BinaryString key = internalMap.keyArray().getString(i);
+            BinaryString value = internalMap.valueArray().getString(i);
+            resultMap.put(key, value);
+        }
+        return resultMap;
+    }
 }
