@@ -21,10 +21,13 @@ import org.apache.flink.cdc.common.event.AddColumnEvent;
 import org.apache.flink.cdc.common.event.AlterColumnTypeEvent;
 import org.apache.flink.cdc.common.event.CreateTableEvent;
 import org.apache.flink.cdc.common.event.DropColumnEvent;
+import org.apache.flink.cdc.common.event.DropTableEvent;
 import org.apache.flink.cdc.common.event.RenameColumnEvent;
 import org.apache.flink.cdc.common.event.SchemaChangeEvent;
 import org.apache.flink.cdc.common.event.SchemaChangeEventType;
 import org.apache.flink.cdc.common.event.TableId;
+import org.apache.flink.cdc.common.event.TruncateTableEvent;
+import org.apache.flink.cdc.common.event.visitor.SchemaChangeEventVisitor;
 import org.apache.flink.cdc.common.exceptions.SchemaEvolveException;
 import org.apache.flink.cdc.common.exceptions.UnsupportedSchemaChangeEventException;
 import org.apache.flink.cdc.common.schema.Column;
@@ -32,7 +35,6 @@ import org.apache.flink.cdc.common.sink.MetadataApplier;
 
 import org.apache.flink.shaded.guava31.com.google.common.collect.Sets;
 
-import com.starrocks.connector.flink.catalog.StarRocksCatalog;
 import com.starrocks.connector.flink.catalog.StarRocksCatalogException;
 import com.starrocks.connector.flink.catalog.StarRocksColumn;
 import com.starrocks.connector.flink.catalog.StarRocksTable;
@@ -52,14 +54,14 @@ public class StarRocksMetadataApplier implements MetadataApplier {
 
     private static final Logger LOG = LoggerFactory.getLogger(StarRocksMetadataApplier.class);
 
-    private final StarRocksCatalog catalog;
+    private final StarRocksEnrichedCatalog catalog;
     private final TableCreateConfig tableCreateConfig;
     private final SchemaChangeConfig schemaChangeConfig;
     private boolean isOpened;
     private Set<SchemaChangeEventType> enabledSchemaEvolutionTypes;
 
     public StarRocksMetadataApplier(
-            StarRocksCatalog catalog,
+            StarRocksEnrichedCatalog catalog,
             TableCreateConfig tableCreateConfig,
             SchemaChangeConfig schemaChangeConfig) {
         this.catalog = catalog;
@@ -86,7 +88,9 @@ public class StarRocksMetadataApplier implements MetadataApplier {
         return Sets.newHashSet(
                 SchemaChangeEventType.CREATE_TABLE,
                 SchemaChangeEventType.ADD_COLUMN,
-                SchemaChangeEventType.DROP_COLUMN);
+                SchemaChangeEventType.DROP_COLUMN,
+                SchemaChangeEventType.DROP_TABLE,
+                SchemaChangeEventType.TRUNCATE_TABLE);
     }
 
     @Override
@@ -97,19 +101,36 @@ public class StarRocksMetadataApplier implements MetadataApplier {
             catalog.open();
         }
 
-        if (schemaChangeEvent instanceof CreateTableEvent) {
-            applyCreateTable((CreateTableEvent) schemaChangeEvent);
-        } else if (schemaChangeEvent instanceof AddColumnEvent) {
-            applyAddColumn((AddColumnEvent) schemaChangeEvent);
-        } else if (schemaChangeEvent instanceof DropColumnEvent) {
-            applyDropColumn((DropColumnEvent) schemaChangeEvent);
-        } else if (schemaChangeEvent instanceof RenameColumnEvent) {
-            applyRenameColumn((RenameColumnEvent) schemaChangeEvent);
-        } else if (schemaChangeEvent instanceof AlterColumnTypeEvent) {
-            applyAlterColumn((AlterColumnTypeEvent) schemaChangeEvent);
-        } else {
-            throw new UnsupportedSchemaChangeEventException(schemaChangeEvent);
-        }
+        SchemaChangeEventVisitor.visit(
+                schemaChangeEvent,
+                addColumnEvent -> {
+                    applyAddColumn(addColumnEvent);
+                    return null;
+                },
+                alterColumnTypeEvent -> {
+                    applyAlterColumnType(alterColumnTypeEvent);
+                    return null;
+                },
+                createTableEvent -> {
+                    applyCreateTable(createTableEvent);
+                    return null;
+                },
+                dropColumnEvent -> {
+                    applyDropColumn(dropColumnEvent);
+                    return null;
+                },
+                dropTableEvent -> {
+                    applyDropTable(dropTableEvent);
+                    return null;
+                },
+                renameColumnEvent -> {
+                    applyRenameColumn(renameColumnEvent);
+                    return null;
+                },
+                truncateTableEvent -> {
+                    applyTruncateTable(truncateTableEvent);
+                    return null;
+                });
     }
 
     private void applyCreateTable(CreateTableEvent createTableEvent) throws SchemaEvolveException {
@@ -289,7 +310,7 @@ public class StarRocksMetadataApplier implements MetadataApplier {
         throw new UnsupportedSchemaChangeEventException(renameColumnEvent);
     }
 
-    private void applyAlterColumn(AlterColumnTypeEvent alterColumnTypeEvent)
+    private void applyAlterColumnType(AlterColumnTypeEvent alterColumnTypeEvent)
             throws SchemaEvolveException {
         // TODO There are limitations for data type conversions. We should know the data types
         // before and after changing so that we can make a validation. But the event only contains
@@ -299,4 +320,24 @@ public class StarRocksMetadataApplier implements MetadataApplier {
         // the alter after a discussion.
         throw new UnsupportedSchemaChangeEventException(alterColumnTypeEvent);
     }
+
+    private void applyTruncateTable(TruncateTableEvent truncateTableEvent) {
+        try {
+            catalog.truncateTable(
+                    truncateTableEvent.tableId().getSchemaName(),
+                    truncateTableEvent.tableId().getTableName());
+        } catch (StarRocksCatalogException e) {
+            throw new SchemaEvolveException(truncateTableEvent, e.getMessage(), e);
+        }
+    }
+
+    private void applyDropTable(DropTableEvent dropTableEvent) {
+        try {
+            catalog.dropTable(
+                    dropTableEvent.tableId().getSchemaName(),
+                    dropTableEvent.tableId().getTableName());
+        } catch (StarRocksCatalogException e) {
+            throw new SchemaEvolveException(dropTableEvent, e.getMessage(), e);
+        }
+    }
 }
