@@ -115,7 +115,7 @@ public class MongoDBRegexFilterITCase extends MongoDBSourceTestBase {
                     String.format("+I[%s, coll_a2, A202]", db1)
                 };
 
-        List<String> actual = TestValuesTableFactory.getResults("mongodb_sink");
+        List<String> actual = TestValuesTableFactory.getResultsAsStrings("mongodb_sink");
         assertThat(actual, containsInAnyOrder(expected));
 
         result.getJobClient().get().cancel().get();
@@ -169,7 +169,7 @@ public class MongoDBRegexFilterITCase extends MongoDBSourceTestBase {
                     String.format("+I[%s, coll_b2, B202]", db1),
                 };
 
-        List<String> actual = TestValuesTableFactory.getResults("mongodb_sink");
+        List<String> actual = TestValuesTableFactory.getResultsAsStrings("mongodb_sink");
         assertThat(actual, containsInAnyOrder(expected));
 
         result.getJobClient().get().cancel().get();
@@ -208,7 +208,7 @@ public class MongoDBRegexFilterITCase extends MongoDBSourceTestBase {
                     String.format("+I[%s, coll_b2, B202]", db0)
                 };
 
-        List<String> actual = TestValuesTableFactory.getResults("mongodb_sink");
+        List<String> actual = TestValuesTableFactory.getResultsAsStrings("mongodb_sink");
         assertThat(actual, containsInAnyOrder(expected));
 
         result.getJobClient().get().cancel().get();
@@ -247,7 +247,7 @@ public class MongoDBRegexFilterITCase extends MongoDBSourceTestBase {
                     String.format("+I[%s, coll_b2, B202]", db0)
                 };
 
-        List<String> actual = TestValuesTableFactory.getResults("mongodb_sink");
+        List<String> actual = TestValuesTableFactory.getResultsAsStrings("mongodb_sink");
         assertThat(actual, containsInAnyOrder(expected));
 
         result.getJobClient().get().cancel().get();
@@ -267,7 +267,7 @@ public class MongoDBRegexFilterITCase extends MongoDBSourceTestBase {
         // 3. Check results
         String[] expected = new String[] {String.format("+I[%s, coll-a1, A101]", db0)};
 
-        List<String> actual = TestValuesTableFactory.getResults("mongodb_sink");
+        List<String> actual = TestValuesTableFactory.getResultsAsStrings("mongodb_sink");
         assertThat(actual, containsInAnyOrder(expected));
 
         result.getJobClient().get().cancel().get();
@@ -292,7 +292,7 @@ public class MongoDBRegexFilterITCase extends MongoDBSourceTestBase {
                     String.format("+I[%s, coll.name, A103]", db)
                 };
 
-        List<String> actual = TestValuesTableFactory.getResults("mongodb_sink");
+        List<String> actual = TestValuesTableFactory.getResultsAsStrings("mongodb_sink");
         assertThat(actual, containsInAnyOrder(expected));
 
         result.getJobClient().get().cancel().get();
