@@ -20,6 +20,7 @@ package org.apache.flink.cdc.connectors.mysql.debezium.reader;
 import org.apache.flink.cdc.common.annotation.VisibleForTesting;
 import org.apache.flink.cdc.connectors.mysql.debezium.task.MySqlBinlogSplitReadTask;
 import org.apache.flink.cdc.connectors.mysql.debezium.task.context.StatefulTaskContext;
+import org.apache.flink.cdc.connectors.mysql.source.config.MySqlSourceConfig;
 import org.apache.flink.cdc.connectors.mysql.source.offset.BinlogOffset;
 import org.apache.flink.cdc.connectors.mysql.source.split.FinishedSnapshotSplitInfo;
 import org.apache.flink.cdc.connectors.mysql.source.split.MySqlBinlogSplit;
@@ -40,7 +41,6 @@ import io.debezium.connector.base.ChangeEventQueue;
 import io.debezium.connector.mysql.MySqlStreamingChangeEventSourceMetrics;
 import io.debezium.pipeline.DataChangeEvent;
 import io.debezium.relational.TableId;
-import io.debezium.relational.Tables;
 import org.apache.kafka.connect.data.Struct;
 import org.apache.kafka.connect.source.SourceRecord;
 import org.slf4j.Logger;
@@ -54,6 +54,7 @@ import java.util.HashSet;
 import java.util.Iterator;
 import java.util.List;
 import java.util.Map;
+import java.util.Optional;
 import java.util.Set;
 import java.util.concurrent.ExecutorService;
 import java.util.concurrent.Executors;
@@ -61,6 +62,9 @@ import java.util.concurrent.ThreadFactory;
 import java.util.concurrent.TimeUnit;
 import java.util.function.Predicate;
 
+import static org.apache.flink.cdc.connectors.mysql.debezium.DebeziumUtils.createBinaryClient;
+import static org.apache.flink.cdc.connectors.mysql.debezium.DebeziumUtils.createMySqlConnection;
+
 /**
  * A Debezium binlog reader implementation that also support reads binlog and filter overlapping
  * snapshot data that {@link SnapshotSplitReader} read.
@@ -81,27 +85,38 @@ public class BinlogSplitReader implements DebeziumReader<SourceRecords, MySqlSpl
     // tableId -> the max splitHighWatermark
     private Map<TableId, BinlogOffset> maxSplitHighWatermarkMap;
     private final Set<TableId> pureBinlogPhaseTables;
-    private Tables.TableFilter capturedTableFilter;
+    private Predicate capturedTableFilter;
     private final StoppableChangeEventSourceContext changeEventSourceContext =
             new StoppableChangeEventSourceContext();
+    private final boolean isParsingOnLineSchemaChanges;
 
     private static final long READER_CLOSE_TIMEOUT = 30L;
 
-    public BinlogSplitReader(StatefulTaskContext statefulTaskContext, int subTaskId) {
+    public BinlogSplitReader(MySqlSourceConfig sourceConfig, int subtaskId) {
+        this(
+                new StatefulTaskContext(
+                        sourceConfig,
+                        createBinaryClient(sourceConfig.getDbzConfiguration()),
+                        createMySqlConnection(sourceConfig)),
+                subtaskId);
+    }
+
+    public BinlogSplitReader(StatefulTaskContext statefulTaskContext, int subtaskId) {
         this.statefulTaskContext = statefulTaskContext;
         ThreadFactory threadFactory =
-                new ThreadFactoryBuilder().setNameFormat("binlog-reader-" + subTaskId).build();
+                new ThreadFactoryBuilder().setNameFormat("binlog-reader-" + subtaskId).build();
         this.executorService = Executors.newSingleThreadExecutor(threadFactory);
         this.currentTaskRunning = true;
         this.pureBinlogPhaseTables = new HashSet<>();
+        this.isParsingOnLineSchemaChanges =
+                statefulTaskContext.getSourceConfig().isParseOnLineSchemaChanges();
     }
 
     public void submitSplit(MySqlSplit mySqlSplit) {
         this.currentBinlogSplit = mySqlSplit.asBinlogSplit();
         configureFilter();
         statefulTaskContext.configure(currentBinlogSplit);
-        this.capturedTableFilter =
-                statefulTaskContext.getConnectorConfig().getTableFilters().dataCollectionFilter();
+        this.capturedTableFilter = statefulTaskContext.getSourceConfig().getTableFilter();
         this.queue = statefulTaskContext.getQueue();
         this.binlogSplitReadTask =
                 new MySqlBinlogSplitReadTask(
@@ -150,6 +165,14 @@ public class BinlogSplitReader implements DebeziumReader<SourceRecords, MySqlSpl
         if (currentTaskRunning) {
             List<DataChangeEvent> batch = queue.poll();
             for (DataChangeEvent event : batch) {
+                if (isParsingOnLineSchemaChanges) {
+                    Optional<SourceRecord> oscRecord =
+                            parseOnLineSchemaChangeEvent(event.getRecord());
+                    if (oscRecord.isPresent()) {
+                        sourceRecords.add(oscRecord.get());
+                        continue;
+                    }
+                }
                 if (shouldEmit(event.getRecord())) {
                     sourceRecords.add(event.getRecord());
                 }
@@ -175,14 +198,10 @@ public class BinlogSplitReader implements DebeziumReader<SourceRecords, MySqlSpl
     @Override
     public void close() {
         try {
-            if (statefulTaskContext.getConnection() != null) {
-                statefulTaskContext.getConnection().close();
-            }
-            if (statefulTaskContext.getBinaryLogClient() != null) {
-                statefulTaskContext.getBinaryLogClient().disconnect();
-            }
-
             stopBinlogReadTask();
+            if (statefulTaskContext != null) {
+                statefulTaskContext.close();
+            }
             if (executorService != null) {
                 executorService.shutdown();
                 if (!executorService.awaitTermination(READER_CLOSE_TIMEOUT, TimeUnit.SECONDS)) {
@@ -191,12 +210,25 @@ public class BinlogSplitReader implements DebeziumReader<SourceRecords, MySqlSpl
                             READER_CLOSE_TIMEOUT);
                 }
             }
-            statefulTaskContext.getDatabaseSchema().close();
         } catch (Exception e) {
             LOG.error("Close binlog reader error", e);
         }
     }
 
+    private Optional<SourceRecord> parseOnLineSchemaChangeEvent(SourceRecord sourceRecord) {
+        if (RecordUtils.isOnLineSchemaChangeEvent(sourceRecord)) {
+            // This is a gh-ost initialized schema change event and should be emitted if the
+            // peeled tableId matches the predicate.
+            TableId originalTableId = RecordUtils.getTableId(sourceRecord);
+            TableId peeledTableId = RecordUtils.peelTableId(originalTableId);
+            if (capturedTableFilter.test(peeledTableId)) {
+                return Optional.of(
+                        RecordUtils.setTableId(sourceRecord, originalTableId, peeledTableId));
+            }
+        }
+        return Optional.empty();
+    }
+
     /**
      * Returns the record should emit or not.
      *
@@ -228,7 +260,8 @@ public class BinlogSplitReader implements DebeziumReader<SourceRecords, MySqlSpl
                 RowType splitKeyType =
                         ChunkUtils.getChunkKeyColumnType(
                                 statefulTaskContext.getDatabaseSchema().tableFor(tableId),
-                                statefulTaskContext.getSourceConfig().getChunkKeyColumns());
+                                statefulTaskContext.getSourceConfig().getChunkKeyColumns(),
+                                statefulTaskContext.getSourceConfig().isTreatTinyInt1AsBoolean());
 
                 Struct target = RecordUtils.getStructContainsChunkKey(sourceRecord);
                 Object[] chunkKey =
@@ -247,7 +280,7 @@ public class BinlogSplitReader implements DebeziumReader<SourceRecords, MySqlSpl
         } else if (RecordUtils.isSchemaChangeEvent(sourceRecord)) {
             if (RecordUtils.isTableChangeRecord(sourceRecord)) {
                 TableId tableId = RecordUtils.getTableId(sourceRecord);
-                return capturedTableFilter.isIncluded(tableId);
+                return capturedTableFilter.test(tableId);
             } else {
                 // Not related to changes in table structure, like `CREATE/DROP DATABASE`, skip it
                 return false;
@@ -270,7 +303,7 @@ public class BinlogSplitReader implements DebeziumReader<SourceRecords, MySqlSpl
         if (!statefulTaskContext.getSourceConfig().isScanNewlyAddedTableEnabled()) {
             // the new added sharding table without history records
             return !maxSplitHighWatermarkMap.containsKey(tableId)
-                    && capturedTableFilter.isIncluded(tableId);
+                    && capturedTableFilter.test(tableId);
         }
         return false;
     }
@@ -280,7 +313,7 @@ public class BinlogSplitReader implements DebeziumReader<SourceRecords, MySqlSpl
                 currentBinlogSplit.getFinishedSnapshotSplitInfos();
         Map<TableId, List<FinishedSnapshotSplitInfo>> splitsInfoMap = new HashMap<>();
         Map<TableId, BinlogOffset> tableIdBinlogPositionMap = new HashMap<>();
-        // specific offset mode
+        // startup mode which is stream only
         if (finishedSplitInfos.isEmpty()) {
             for (TableId tableId : currentBinlogSplit.getTableSchemas().keySet()) {
                 tableIdBinlogPositionMap.put(tableId, currentBinlogSplit.getStartingOffset());
