@@ -17,6 +17,7 @@
 
 package org.apache.flink.cdc.connectors.mysql.source.assigners;
 
+import org.apache.flink.api.connector.source.SplitEnumeratorContext;
 import org.apache.flink.cdc.connectors.mysql.debezium.DebeziumUtils;
 import org.apache.flink.cdc.connectors.mysql.schema.MySqlSchema;
 import org.apache.flink.cdc.connectors.mysql.source.assigners.state.ChunkSplitterState;
@@ -79,6 +80,7 @@ public class MySqlSnapshotSplitAssigner implements MySqlSplitAssigner {
     private final int currentParallelism;
     private final List<TableId> remainingTables;
     private final boolean isRemainingTablesCheckpointed;
+    private final SplitEnumeratorContext<MySqlSplit> enumeratorContext;
 
     private final MySqlPartition partition;
     private final Object lock = new Object();
@@ -95,7 +97,8 @@ public class MySqlSnapshotSplitAssigner implements MySqlSplitAssigner {
             MySqlSourceConfig sourceConfig,
             int currentParallelism,
             List<TableId> remainingTables,
-            boolean isTableIdCaseSensitive) {
+            boolean isTableIdCaseSensitive,
+            SplitEnumeratorContext<MySqlSplit> enumeratorContext) {
         this(
                 sourceConfig,
                 currentParallelism,
@@ -108,13 +111,15 @@ public class MySqlSnapshotSplitAssigner implements MySqlSplitAssigner {
                 remainingTables,
                 isTableIdCaseSensitive,
                 true,
-                ChunkSplitterState.NO_SPLITTING_TABLE_STATE);
+                ChunkSplitterState.NO_SPLITTING_TABLE_STATE,
+                enumeratorContext);
     }
 
     public MySqlSnapshotSplitAssigner(
             MySqlSourceConfig sourceConfig,
             int currentParallelism,
-            SnapshotPendingSplitsState checkpoint) {
+            SnapshotPendingSplitsState checkpoint,
+            SplitEnumeratorContext<MySqlSplit> enumeratorContext) {
         this(
                 sourceConfig,
                 currentParallelism,
@@ -127,7 +132,8 @@ public class MySqlSnapshotSplitAssigner implements MySqlSplitAssigner {
                 checkpoint.getRemainingTables(),
                 checkpoint.isTableIdCaseSensitive(),
                 checkpoint.isRemainingTablesCheckpointed(),
-                checkpoint.getChunkSplitterState());
+                checkpoint.getChunkSplitterState(),
+                enumeratorContext);
     }
 
     private MySqlSnapshotSplitAssigner(
@@ -142,7 +148,8 @@ public class MySqlSnapshotSplitAssigner implements MySqlSplitAssigner {
             List<TableId> remainingTables,
             boolean isTableIdCaseSensitive,
             boolean isRemainingTablesCheckpointed,
-            ChunkSplitterState chunkSplitterState) {
+            ChunkSplitterState chunkSplitterState,
+            SplitEnumeratorContext<MySqlSplit> enumeratorContext) {
         this.sourceConfig = sourceConfig;
         this.currentParallelism = currentParallelism;
         this.alreadyProcessedTables = alreadyProcessedTables;
@@ -168,10 +175,12 @@ public class MySqlSnapshotSplitAssigner implements MySqlSplitAssigner {
                 createChunkSplitter(sourceConfig, isTableIdCaseSensitive, chunkSplitterState);
         this.partition =
                 new MySqlPartition(sourceConfig.getMySqlConnectorConfig().getLogicalName());
+        this.enumeratorContext = enumeratorContext;
     }
 
     @Override
     public void open() {
+        shouldEnterProcessingBacklog();
         chunkSplitter.open();
         discoveryCaptureTables();
         captureNewlyAddedTables();
@@ -397,17 +406,20 @@ public class MySqlSnapshotSplitAssigner implements MySqlSplitAssigner {
     @Override
     public void onFinishedSplits(Map<String, BinlogOffset> splitFinishedOffsets) {
         this.splitFinishedOffsets.putAll(splitFinishedOffsets);
-        if (allSnapshotSplitsFinished()
-                && AssignerStatus.isAssigningSnapshotSplits(assignerStatus)) {
-            // Skip the waiting checkpoint when current parallelism is 1 which means we do not need
-            // to care about the global output data order of snapshot splits and binlog split.
-            if (currentParallelism == 1) {
-                assignerStatus = assignerStatus.onFinish();
-                LOG.info(
-                        "Snapshot split assigner received all splits finished and the job parallelism is 1, snapshot split assigner is turn into finished status.");
-            } else {
-                LOG.info(
-                        "Snapshot split assigner received all splits finished, waiting for a complete checkpoint to mark the assigner finished.");
+        if (allSnapshotSplitsFinished()) {
+            enumeratorContext.setIsProcessingBacklog(false);
+            if (AssignerStatus.isAssigningSnapshotSplits(assignerStatus)) {
+                // Skip the waiting checkpoint when current parallelism is 1 which means we do not
+                // need
+                // to care about the global output data order of snapshot splits and binlog split.
+                if (currentParallelism == 1) {
+                    assignerStatus = assignerStatus.onFinish();
+                    LOG.info(
+                            "Snapshot split assigner received all splits finished and the job parallelism is 1, snapshot split assigner is turn into finished status.");
+                } else {
+                    LOG.info(
+                            "Snapshot split assigner received all splits finished, waiting for a complete checkpoint to mark the assigner finished.");
+                }
             }
         }
     }
@@ -471,7 +483,7 @@ public class MySqlSnapshotSplitAssigner implements MySqlSplitAssigner {
     public void startAssignNewlyAddedTables() {
         Preconditions.checkState(
                 AssignerStatus.isAssigningFinished(assignerStatus),
-                "Invalid assigner status {}",
+                "Invalid assigner status %s",
                 assignerStatus);
         assignerStatus = assignerStatus.startAssignNewlyTables();
     }
@@ -480,7 +492,7 @@ public class MySqlSnapshotSplitAssigner implements MySqlSplitAssigner {
     public void onBinlogSplitUpdated() {
         Preconditions.checkState(
                 AssignerStatus.isNewlyAddedAssigningSnapshotFinished(assignerStatus),
-                "Invalid assigner status {}",
+                "Invalid assigner status %s",
                 assignerStatus);
         assignerStatus = assignerStatus.onBinlogSplitUpdated();
     }
@@ -601,14 +613,16 @@ public class MySqlSnapshotSplitAssigner implements MySqlSplitAssigner {
             return new MySqlChunkSplitter(
                     mySqlSchema,
                     sourceConfig,
-                    tableId != null
-                                    && sourceConfig
-                                            .getTableFilters()
-                                            .dataCollectionFilter()
-                                            .isIncluded(tableId)
+                    tableId != null && sourceConfig.getTableFilter().test(tableId)
                             ? chunkSplitterState
                             : ChunkSplitterState.NO_SPLITTING_TABLE_STATE);
         }
         return new MySqlChunkSplitter(mySqlSchema, sourceConfig);
     }
+
+    private void shouldEnterProcessingBacklog() {
+        if (assignerStatus == AssignerStatus.INITIAL_ASSIGNING) {
+            enumeratorContext.setIsProcessingBacklog(true);
+        }
+    }
 }
