@@ -49,6 +49,7 @@ public class MySqlSourceConfigFactory implements Serializable {
     private ServerIdRange serverIdRange;
     private List<String> databaseList;
     private List<String> tableList;
+    private String excludeTableList;
     private String serverTimeZone = ZoneId.systemDefault().getId();
     private StartupOptions startupOptions = StartupOptions.initial();
     private int splitSize = MySqlSourceOptions.SCAN_INCREMENTAL_SNAPSHOT_CHUNK_SIZE.defaultValue();
@@ -69,6 +70,9 @@ public class MySqlSourceConfigFactory implements Serializable {
     private Properties dbzProperties;
     private Map<ObjectPath, String> chunkKeyColumns = new HashMap<>();
     private boolean skipSnapshotBackfill = false;
+    private boolean parseOnLineSchemaChanges = false;
+    private boolean treatTinyInt1AsBoolean = true;
+    private boolean useLegacyJsonFormat = true;
 
     public MySqlSourceConfigFactory hostname(String hostname) {
         this.hostname = hostname;
@@ -102,6 +106,11 @@ public class MySqlSourceConfigFactory implements Serializable {
         return this;
     }
 
+    public MySqlSourceConfigFactory excludeTableList(String tableInclusions) {
+        this.excludeTableList = tableInclusions;
+        return this;
+    }
+
     /** Name of the MySQL database to use when connecting to the MySQL database server. */
     public MySqlSourceConfigFactory username(String username) {
         this.username = username;
@@ -270,6 +279,15 @@ public class MySqlSourceConfigFactory implements Serializable {
         return this;
     }
 
+    /**
+     * Whether to use legacy json format. The default value is true, which means there is no
+     * whitespace before value and after comma in json format.
+     */
+    public MySqlSourceConfigFactory useLegacyJsonFormat(boolean useLegacyJsonFormat) {
+        this.useLegacyJsonFormat = useLegacyJsonFormat;
+        return this;
+    }
+
     /**
      * Whether to close idle readers at the end of the snapshot phase. This feature depends on
      * FLIP-147: Support Checkpoints After Tasks Finished. The flink version is required to be
@@ -285,6 +303,17 @@ public class MySqlSourceConfigFactory implements Serializable {
         return this;
     }
 
+    /** Whether to parse gh-ost/pt-osc utility generated schema change events. Defaults to false. */
+    public MySqlSourceConfigFactory parseOnLineSchemaChanges(boolean parseOnLineSchemaChanges) {
+        this.parseOnLineSchemaChanges = parseOnLineSchemaChanges;
+        return this;
+    }
+
+    public MySqlSourceConfigFactory treatTinyInt1AsBoolean(boolean treatTinyInt1AsBoolean) {
+        this.treatTinyInt1AsBoolean = treatTinyInt1AsBoolean;
+        return this;
+    }
+
     /** Creates a new {@link MySqlSourceConfig} for the given subtask {@code subtaskId}. */
     public MySqlSourceConfig createConfig(int subtaskId) {
         // hard code server name, because we don't need to distinguish it, docs:
@@ -360,6 +389,7 @@ public class MySqlSourceConfigFactory implements Serializable {
                 password,
                 databaseList,
                 tableList,
+                excludeTableList,
                 serverIdRange,
                 startupOptions,
                 splitSize,
@@ -377,6 +407,9 @@ public class MySqlSourceConfigFactory implements Serializable {
                 props,
                 jdbcProperties,
                 chunkKeyColumns,
-                skipSnapshotBackfill);
+                skipSnapshotBackfill,
+                parseOnLineSchemaChanges,
+                treatTinyInt1AsBoolean,
+                useLegacyJsonFormat);
     }
 }
