@@ -321,14 +321,16 @@ public class PostgresSourceBuilder<T> {
                                     remainingTables,
                                     isTableIdCaseSensitive,
                                     dataSourceDialect,
-                                    offsetFactory);
+                                    offsetFactory,
+                                    enumContext);
                 } catch (Exception e) {
                     throw new FlinkRuntimeException(
                             "Failed to discover captured tables for enumerator", e);
                 }
             } else {
                 splitAssigner =
-                        new StreamSplitAssigner(sourceConfig, dataSourceDialect, offsetFactory);
+                        new StreamSplitAssigner(
+                                sourceConfig, dataSourceDialect, offsetFactory, enumContext);
             }
 
             return new PostgresSourceEnumerator(
@@ -352,14 +354,16 @@ public class PostgresSourceBuilder<T> {
                                 enumContext.currentParallelism(),
                                 (HybridPendingSplitsState) checkpoint,
                                 dataSourceDialect,
-                                offsetFactory);
+                                offsetFactory,
+                                enumContext);
             } else if (checkpoint instanceof StreamPendingSplitsState) {
                 splitAssigner =
                         new StreamSplitAssigner(
                                 sourceConfig,
                                 (StreamPendingSplitsState) checkpoint,
                                 dataSourceDialect,
-                                offsetFactory);
+                                offsetFactory,
+                                enumContext);
             } else {
                 throw new UnsupportedOperationException(
                         "Unsupported restored PendingSplitsState: " + checkpoint);
@@ -385,7 +389,6 @@ public class PostgresSourceBuilder<T> {
             final SourceReaderMetrics sourceReaderMetrics =
                     new SourceReaderMetrics(readerContext.metricGroup());
 
-            sourceReaderMetrics.registerMetrics();
             IncrementalSourceReaderContext incrementalSourceReaderContext =
                     new IncrementalSourceReaderContext(readerContext);
             Supplier<IncrementalSourceSplitReader<JdbcSourceConfig>> splitReaderSupplier =
