@@ -17,7 +17,6 @@
 
 package org.apache.flink.cdc.connectors.postgres.source.fetch;
 
-import org.apache.flink.cdc.connectors.base.relational.JdbcSourceEventDispatcher;
 import org.apache.flink.cdc.connectors.base.source.meta.split.SnapshotSplit;
 import org.apache.flink.cdc.connectors.base.source.meta.split.StreamSplit;
 import org.apache.flink.cdc.connectors.base.source.reader.external.AbstractScanFetchTask;
@@ -28,6 +27,7 @@ import org.apache.flink.cdc.connectors.postgres.source.utils.PostgresQueryUtils;
 import org.apache.flink.util.FlinkRuntimeException;
 
 import io.debezium.connector.postgresql.PostgresConnectorConfig;
+import io.debezium.connector.postgresql.PostgresEventDispatcher;
 import io.debezium.connector.postgresql.PostgresOffsetContext;
 import io.debezium.connector.postgresql.PostgresPartition;
 import io.debezium.connector.postgresql.PostgresSchema;
@@ -102,7 +102,7 @@ public class PostgresScanFetchTask extends AbstractScanFetchTask {
                         ctx.getDbzConnectorConfig(),
                         ctx.getDatabaseSchema(),
                         ctx.getOffsetContext(),
-                        ctx.getDispatcher(),
+                        ctx.getEventDispatcher(),
                         ctx.getSnapshotChangeEventSourceMetrics(),
                         snapshotSplit);
 
@@ -135,8 +135,8 @@ public class PostgresScanFetchTask extends AbstractScanFetchTask {
                         ctx.getDbzConnectorConfig(),
                         ctx.getSnapShotter(),
                         ctx.getConnection(),
-                        ctx.getDispatcher(),
-                        ctx.getPostgresDispatcher(),
+                        ctx.getEventDispatcher(),
+                        ctx.getWaterMarkDispatcher(),
                         ctx.getErrorHandler(),
                         ctx.getTaskContext().getClock(),
                         ctx.getDatabaseSchema(),
@@ -215,7 +215,7 @@ public class PostgresScanFetchTask extends AbstractScanFetchTask {
 
         private final PostgresConnection jdbcConnection;
         private final PostgresConnectorConfig connectorConfig;
-        private final JdbcSourceEventDispatcher<PostgresPartition> dispatcher;
+        private final PostgresEventDispatcher<TableId> eventDispatcher;
         private final SnapshotSplit snapshotSplit;
         private final PostgresOffsetContext offsetContext;
         private final PostgresSchema databaseSchema;
@@ -227,7 +227,7 @@ public class PostgresScanFetchTask extends AbstractScanFetchTask {
                 PostgresConnectorConfig connectorConfig,
                 PostgresSchema databaseSchema,
                 PostgresOffsetContext previousOffset,
-                JdbcSourceEventDispatcher dispatcher,
+                PostgresEventDispatcher<TableId> eventDispatcher,
                 SnapshotProgressListener snapshotProgressListener,
                 SnapshotSplit snapshotSplit) {
             super(connectorConfig, snapshotProgressListener);
@@ -235,7 +235,7 @@ public class PostgresScanFetchTask extends AbstractScanFetchTask {
             this.connectorConfig = connectorConfig;
             this.snapshotProgressListener = snapshotProgressListener;
             this.databaseSchema = databaseSchema;
-            this.dispatcher = dispatcher;
+            this.eventDispatcher = eventDispatcher;
             this.snapshotSplit = snapshotSplit;
             this.offsetContext = previousOffset;
             this.clock = Clock.SYSTEM;
@@ -260,7 +260,7 @@ public class PostgresScanFetchTask extends AbstractScanFetchTask {
         private void createDataEvents(PostgresSnapshotContext snapshotContext, TableId tableId)
                 throws InterruptedException {
             EventDispatcher.SnapshotReceiver<PostgresPartition> snapshotReceiver =
-                    dispatcher.getSnapshotChangeEventReceiver();
+                    eventDispatcher.getSnapshotChangeEventReceiver();
             LOG.info("Snapshotting table {}", tableId);
             createDataEventsForTable(
                     snapshotContext,
@@ -337,7 +337,7 @@ public class PostgresScanFetchTask extends AbstractScanFetchTask {
                     SnapshotChangeRecordEmitter<PostgresPartition> emitter =
                             new SnapshotChangeRecordEmitter<>(
                                     snapshotContext.partition, snapshotContext.offset, row, clock);
-                    dispatcher.dispatchSnapshotEvent(
+                    eventDispatcher.dispatchSnapshotEvent(
                             snapshotContext.partition, table.id(), emitter, snapshotReceiver);
                 }
                 LOG.info(
