@@ -44,6 +44,7 @@ import org.apache.flink.util.FlinkRuntimeException;
 
 import io.debezium.connector.postgresql.connection.PostgresConnection;
 import io.debezium.jdbc.JdbcConnection;
+import io.debezium.relational.TableId;
 import org.apache.commons.lang3.StringUtils;
 import org.junit.After;
 import org.junit.Before;
@@ -74,6 +75,7 @@ import static org.apache.flink.table.api.DataTypes.BIGINT;
 import static org.apache.flink.table.api.DataTypes.STRING;
 import static org.apache.flink.table.catalog.Column.physical;
 import static org.apache.flink.util.Preconditions.checkState;
+import static org.junit.Assert.assertEquals;
 import static org.junit.Assert.assertTrue;
 
 /** IT tests for {@link PostgresSourceBuilder.PostgresIncrementalSource}. */
@@ -253,9 +255,7 @@ public class PostgresSourceITCase extends PostgresTestBase {
                         PostgresTestUtils.FailoverType.NONE,
                         PostgresTestUtils.FailoverPhase.NEVER,
                         new String[] {"customers_no_pk"},
-                        RestartStrategies.noRestart(),
-                        false,
-                        null);
+                        RestartStrategies.noRestart());
             } catch (Exception e) {
                 assertTrue(
                         ExceptionUtils.findThrowableWithMessage(
@@ -272,9 +272,7 @@ public class PostgresSourceITCase extends PostgresTestBase {
                     PostgresTestUtils.FailoverType.NONE,
                     PostgresTestUtils.FailoverPhase.NEVER,
                     new String[] {"customers_no_pk"},
-                    RestartStrategies.noRestart(),
-                    false,
-                    null);
+                    RestartStrategies.noRestart());
         }
     }
 
@@ -287,8 +285,7 @@ public class PostgresSourceITCase extends PostgresTestBase {
                 PostgresTestUtils.FailoverPhase.SNAPSHOT,
                 new String[] {"customers"},
                 RestartStrategies.fixedDelayRestart(1, 0),
-                true,
-                null);
+                Collections.singletonMap("scan.incremental.snapshot.backfill.skip", "true"));
     }
 
     @Test
@@ -660,8 +657,8 @@ public class PostgresSourceITCase extends PostgresTestBase {
                     PostgresTestUtils.FailoverPhase.NEVER,
                     new String[] {"customers"},
                     RestartStrategies.noRestart(),
-                    false,
-                    chunkColumn);
+                    Collections.singletonMap(
+                            "scan.incremental.snapshot.chunk.key-column", chunkColumn));
         } catch (Exception e) {
             assertTrue(
                     ExceptionUtils.findThrowableWithMessage(
@@ -673,6 +670,34 @@ public class PostgresSourceITCase extends PostgresTestBase {
         }
     }
 
+    @Test
+    public void testHeartBeat() throws Exception {
+        try (PostgresConnection connection = getConnection()) {
+            connection.execute("CREATE TABLE IF NOT EXISTS heart_beat_table(a int)");
+            connection.commit();
+        }
+
+        TableId tableId = new TableId(null, "public", "heart_beat_table");
+        try (PostgresConnection connection = getConnection()) {
+            assertEquals(0, getCountOfTable(connection, tableId));
+        }
+
+        Map<String, String> options = new HashMap<>();
+        options.put("heartbeat.interval.ms", "100");
+        options.put("debezium.heartbeat.action.query", "INSERT INTO heart_beat_table VALUES(1)");
+        testPostgresParallelSource(
+                1,
+                scanStartupMode,
+                PostgresTestUtils.FailoverType.NONE,
+                PostgresTestUtils.FailoverPhase.NEVER,
+                new String[] {"customers"},
+                RestartStrategies.noRestart(),
+                options);
+        try (PostgresConnection connection = getConnection()) {
+            assertTrue(getCountOfTable(connection, tableId) > 0);
+        }
+    }
+
     private List<String> testBackfillWhenWritingEvents(
             boolean skipSnapshotBackfill,
             int fetchSize,
@@ -776,8 +801,25 @@ public class PostgresSourceITCase extends PostgresTestBase {
                 failoverPhase,
                 captureCustomerTables,
                 RestartStrategies.fixedDelayRestart(1, 0),
-                false,
-                null);
+                new HashMap<>());
+    }
+
+    private void testPostgresParallelSource(
+            int parallelism,
+            String scanStartupMode,
+            PostgresTestUtils.FailoverType failoverType,
+            PostgresTestUtils.FailoverPhase failoverPhase,
+            String[] captureCustomerTables,
+            RestartStrategies.RestartStrategyConfiguration restartStrategyConfiguration)
+            throws Exception {
+        testPostgresParallelSource(
+                parallelism,
+                scanStartupMode,
+                failoverType,
+                failoverPhase,
+                captureCustomerTables,
+                restartStrategyConfiguration,
+                new HashMap<>());
     }
 
     private void testPostgresParallelSource(
@@ -787,8 +829,7 @@ public class PostgresSourceITCase extends PostgresTestBase {
             PostgresTestUtils.FailoverPhase failoverPhase,
             String[] captureCustomerTables,
             RestartStrategies.RestartStrategyConfiguration restartStrategyConfiguration,
-            boolean skipSnapshotBackfill,
-            String chunkColumn)
+            Map<String, String> otherOptions)
             throws Exception {
         StreamExecutionEnvironment env = StreamExecutionEnvironment.getExecutionEnvironment();
         StreamTableEnvironment tEnv = StreamTableEnvironment.create(env);
@@ -817,9 +858,8 @@ public class PostgresSourceITCase extends PostgresTestBase {
                                 + " 'scan.startup.mode' = '%s',"
                                 + " 'scan.incremental.snapshot.chunk.size' = '100',"
                                 + " 'slot.name' = '%s',"
-                                + " 'scan.incremental.snapshot.backfill.skip' = '%s',"
                                 + " 'scan.lsn-commit.checkpoints-num-delay' = '1'"
-                                + ""
+                                + "%s"
                                 + ")",
                         customDatabase.getHost(),
                         customDatabase.getDatabasePort(),
@@ -830,12 +870,16 @@ public class PostgresSourceITCase extends PostgresTestBase {
                         getTableNameRegex(captureCustomerTables),
                         scanStartupMode,
                         slotName,
-                        skipSnapshotBackfill,
-                        chunkColumn == null
+                        otherOptions.isEmpty()
                                 ? ""
-                                : ",'scan.incremental.snapshot.chunk.key-column'='"
-                                        + chunkColumn
-                                        + "'");
+                                : ","
+                                        + otherOptions.entrySet().stream()
+                                                .map(
+                                                        e ->
+                                                                String.format(
+                                                                        "'%s'='%s'",
+                                                                        e.getKey(), e.getValue()))
+                                                .collect(Collectors.joining(",")));
         tEnv.executeSql(sourceDDL);
         TableResult tableResult = tEnv.executeSql("select * from customers");
 
@@ -1163,4 +1207,23 @@ public class PostgresSourceITCase extends PostgresTestBase {
         properties.put("dbname", customDatabase.getDatabaseName());
         return createConnection(properties);
     }
+
+    private static long getCountOfTable(JdbcConnection jdbc, TableId tableId) throws SQLException {
+        // The statement used to get approximate row count which is less
+        // accurate than COUNT(*), but is more efficient for large table.
+        // https://stackoverflow.com/questions/7943233/fast-way-to-discover-the-row-count-of-a-table-in-postgresql
+        // NOTE: it requires ANALYZE or VACUUM to be run first in PostgreSQL.
+        final String query = String.format("SELECT COUNT(1) FROM %s", tableId.toString());
+
+        return jdbc.queryAndMap(
+                query,
+                rs -> {
+                    if (!rs.next()) {
+                        throw new SQLException(
+                                String.format(
+                                        "No result returned after running query [%s]", query));
+                    }
+                    return rs.getLong(1);
+                });
+    }
 }
