@@ -165,20 +165,24 @@ public class PostgreSQLConnectorITCase extends PostgresTestBase {
          * The final database table looks like this:
          *
          * > SELECT * FROM products;
-         * +-----+--------------------+---------------------------------------------------------+--------+
-         * | id  | name               | description                                             | weight |
-         * +-----+--------------------+---------------------------------------------------------+--------+
-         * | 101 | scooter            | Small 2-wheel scooter                                   |   3.14 |
-         * | 102 | car battery        | 12V car battery                                         |    8.1 |
-         * | 103 | 12-pack drill bits | 12-pack of drill bits with sizes ranging from #40 to #3 |    0.8 |
-         * | 104 | hammer             | 12oz carpenter's hammer                                 |   0.75 |
-         * | 105 | hammer             | 14oz carpenter's hammer                                 |  0.875 |
-         * | 106 | hammer             | 18oz carpenter hammer                                   |      1 |
-         * | 107 | rocks              | box of assorted rocks                                   |    5.1 |
-         * | 108 | jacket             | water resistent black wind breaker                      |    0.1 |
-         * | 109 | spare tire         | 24 inch spare tire                                      |   22.2 |
-         * | 110 | jacket             | new water resistent white wind breaker                  |    0.5 |
-         * +-----+--------------------+---------------------------------------------------------+--------+
+         * +-----+--------------------+-------------------------------------------------
+         * --------+--------+
+         * | id | name | description | weight |
+         * +-----+--------------------+-------------------------------------------------
+         * --------+--------+
+         * | 101 | scooter | Small 2-wheel scooter | 3.14 |
+         * | 102 | car battery | 12V car battery | 8.1 |
+         * | 103 | 12-pack drill bits | 12-pack of drill bits with sizes ranging from
+         * #40 to #3 | 0.8 |
+         * | 104 | hammer | 12oz carpenter's hammer | 0.75 |
+         * | 105 | hammer | 14oz carpenter's hammer | 0.875 |
+         * | 106 | hammer | 18oz carpenter hammer | 1 |
+         * | 107 | rocks | box of assorted rocks | 5.1 |
+         * | 108 | jacket | water resistent black wind breaker | 0.1 |
+         * | 109 | spare tire | 24 inch spare tire | 22.2 |
+         * | 110 | jacket | new water resistent white wind breaker | 0.5 |
+         * +-----+--------------------+-------------------------------------------------
+         * --------+--------+
          * </pre>
          */
 
@@ -193,7 +197,7 @@ public class PostgreSQLConnectorITCase extends PostgresTestBase {
                     "spare tire,22.200"
                 };
 
-        List<String> actual = TestValuesTableFactory.getResults("sink");
+        List<String> actual = TestValuesTableFactory.getResultsAsStrings("sink");
         assertThat(actual, containsInAnyOrder(expected));
 
         result.getJobClient().get().cancel().get();
@@ -246,7 +250,8 @@ public class PostgreSQLConnectorITCase extends PostgresTestBase {
 
         // async submit job
         TableResult result = tEnv.executeSql("INSERT INTO sink SELECT * FROM debezium_source");
-        // wait for the source startup, we don't have a better way to wait it, use sleep for now
+        // wait for the source startup, we don't have a better way to wait it, use sleep
+        // for now
         Thread.sleep(10000L);
 
         try (Connection connection = getJdbcConnection(POSTGRES_CONTAINER);
@@ -266,7 +271,7 @@ public class PostgreSQLConnectorITCase extends PostgresTestBase {
         String[] expected =
                 new String[] {"110,jacket,new water resistent white wind breaker,0.500"};
 
-        List<String> actual = TestValuesTableFactory.getResults("sink");
+        List<String> actual = TestValuesTableFactory.getResultsAsStrings("sink");
         assertThat(actual, containsInAnyOrder(expected));
 
         result.getJobClient().get().cancel().get();
@@ -454,7 +459,7 @@ public class PostgreSQLConnectorITCase extends PostgresTestBase {
                         "+I(1,[50],32767,65535,2147483647,5.5,6.6,123.12345,404.4,true,Hello World,a,abc,abcd..xyz,2020-07-17T18:00:22.123,2020-07-17T18:00:22.123456,2020-07-17,18:00:22,500,{\"hexewkb\":\"0105000020e610000001000000010200000002000000a779c7293a2465400b462575025a46c0c66d3480b7fc6440c3d32b65195246c0\",\"srid\":4326},{\"hexewkb\":\"0101000020730c00001c7c613255de6540787aa52c435c42c0\",\"srid\":3187})",
                         "-D(1,[50],32767,65535,2147483647,5.5,6.6,123.12345,404.4,true,Hello World,a,abc,abcd..xyz,2020-07-17T18:00:22.123,2020-07-17T18:00:22.123456,2020-07-17,18:00:22,500,{\"hexewkb\":\"0105000020e610000001000000010200000002000000a779c7293a2465400b462575025a46c0c66d3480b7fc6440c3d32b65195246c0\",\"srid\":4326},{\"hexewkb\":\"0101000020730c00001c7c613255de6540787aa52c435c42c0\",\"srid\":3187})",
                         "+I(1,[50],0,65535,2147483647,5.5,6.6,123.12345,404.4,true,Hello World,a,abc,abcd..xyz,2020-07-17T18:00:22.123,2020-07-17T18:00:22.123456,2020-07-17,18:00:22,500,{\"hexewkb\":\"0105000020e610000001000000010200000002000000a779c7293a2465400b462575025a46c0c66d3480b7fc6440c3d32b65195246c0\",\"srid\":4326},{\"hexewkb\":\"0101000020730c00001c7c613255de6540787aa52c435c42c0\",\"srid\":3187})");
-        List<String> actual = TestValuesTableFactory.getRawResults("sink");
+        List<String> actual = TestValuesTableFactory.getRawResultsAsStrings("sink");
         assertEquals(expected, actual);
 
         result.getJobClient().get().cancel().get();
@@ -469,6 +474,7 @@ public class PostgreSQLConnectorITCase extends PostgresTestBase {
                                 + " db_name STRING METADATA FROM 'database_name' VIRTUAL,"
                                 + " schema_name STRING METADATA VIRTUAL,"
                                 + " table_name STRING METADATA VIRTUAL,"
+                                + " row_kind STRING METADATA FROM 'row_kind' VIRTUAL,"
                                 + " id INT NOT NULL,"
                                 + " name STRING,"
                                 + " description STRING,"
@@ -501,6 +507,7 @@ public class PostgreSQLConnectorITCase extends PostgresTestBase {
                         + " database_name STRING,"
                         + " schema_name STRING,"
                         + " table_name STRING,"
+                        + " row_kind STRING,"
                         + " id INT,"
                         + " name STRING,"
                         + " description STRING,"
@@ -546,53 +553,53 @@ public class PostgreSQLConnectorITCase extends PostgresTestBase {
                 Arrays.asList(
                         "+I("
                                 + databaseName
-                                + ",inventory,products,101,scooter,Small 2-wheel scooter,3.140)",
+                                + ",inventory,products,+I,101,scooter,Small 2-wheel scooter,3.140)",
                         "+I("
                                 + databaseName
-                                + ",inventory,products,102,car battery,12V car battery,8.100)",
+                                + ",inventory,products,+I,102,car battery,12V car battery,8.100)",
                         "+I("
                                 + databaseName
-                                + ",inventory,products,103,12-pack drill bits,12-pack of drill bits with sizes ranging from #40 to #3,0.800)",
+                                + ",inventory,products,+I,103,12-pack drill bits,12-pack of drill bits with sizes ranging from #40 to #3,0.800)",
                         "+I("
                                 + databaseName
-                                + ",inventory,products,104,hammer,12oz carpenter's hammer,0.750)",
+                                + ",inventory,products,+I,104,hammer,12oz carpenter's hammer,0.750)",
                         "+I("
                                 + databaseName
-                                + ",inventory,products,105,hammer,14oz carpenter's hammer,0.875)",
+                                + ",inventory,products,+I,105,hammer,14oz carpenter's hammer,0.875)",
                         "+I("
                                 + databaseName
-                                + ",inventory,products,106,hammer,16oz carpenter's hammer,1.000)",
+                                + ",inventory,products,+I,106,hammer,16oz carpenter's hammer,1.000)",
                         "+I("
                                 + databaseName
-                                + ",inventory,products,107,rocks,box of assorted rocks,5.300)",
+                                + ",inventory,products,+I,107,rocks,box of assorted rocks,5.300)",
                         "+I("
                                 + databaseName
-                                + ",inventory,products,108,jacket,water resistent black wind breaker,0.100)",
+                                + ",inventory,products,+I,108,jacket,water resistent black wind breaker,0.100)",
                         "+I("
                                 + databaseName
-                                + ",inventory,products,109,spare tire,24 inch spare tire,22.200)",
+                                + ",inventory,products,+I,109,spare tire,24 inch spare tire,22.200)",
                         "+I("
                                 + databaseName
-                                + ",inventory,products,110,jacket,water resistent white wind breaker,0.200)",
+                                + ",inventory,products,+I,110,jacket,water resistent white wind breaker,0.200)",
                         "+I("
                                 + databaseName
-                                + ",inventory,products,111,scooter,Big 2-wheel scooter ,5.180)",
+                                + ",inventory,products,+I,111,scooter,Big 2-wheel scooter ,5.180)",
                         "+U("
                                 + databaseName
-                                + ",inventory,products,106,hammer,18oz carpenter hammer,1.000)",
+                                + ",inventory,products,+U,106,hammer,18oz carpenter hammer,1.000)",
                         "+U("
                                 + databaseName
-                                + ",inventory,products,107,rocks,box of assorted rocks,5.100)",
+                                + ",inventory,products,+U,107,rocks,box of assorted rocks,5.100)",
                         "+U("
                                 + databaseName
-                                + ",inventory,products,110,jacket,new water resistent white wind breaker,0.500)",
+                                + ",inventory,products,+U,110,jacket,new water resistent white wind breaker,0.500)",
                         "+U("
                                 + databaseName
-                                + ",inventory,products,111,scooter,Big 2-wheel scooter ,5.170)",
+                                + ",inventory,products,+U,111,scooter,Big 2-wheel scooter ,5.170)",
                         "-D("
                                 + databaseName
-                                + ",inventory,products,111,scooter,Big 2-wheel scooter ,5.170)");
-        List<String> actual = TestValuesTableFactory.getRawResults("sink");
+                                + ",inventory,products,-D,111,scooter,Big 2-wheel scooter ,5.170)");
+        List<String> actual = TestValuesTableFactory.getRawResultsAsStrings("sink");
         Collections.sort(actual);
         Collections.sort(expected);
         assertEquals(expected, actual);
@@ -679,20 +686,24 @@ public class PostgreSQLConnectorITCase extends PostgresTestBase {
          * The final database table looks like this:
          *
          * > SELECT * FROM products;
-         * +-----+--------------------+---------------------------------------------------------+--------+
-         * | id  | name               | description                                             | weight |
-         * +-----+--------------------+---------------------------------------------------------+--------+
-         * | 101 | scooter            | Small 2-wheel scooter                                   |   3.14 |
-         * | 102 | car battery        | 12V car battery                                         |    8.1 |
-         * | 103 | 12-pack drill bits | 12-pack of drill bits with sizes ranging from #40 to #3 |    0.8 |
-         * | 104 | hammer             | 12oz carpenter's hammer                                 |   0.75 |
-         * | 105 | hammer             | 14oz carpenter's hammer                                 |  0.875 |
-         * | 106 | hammer             | 18oz carpenter hammer                                   |      1 |
-         * | 107 | rocks              | box of assorted rocks                                   |    5.1 |
-         * | 108 | jacket             | water resistent black wind breaker                      |    0.1 |
-         * | 109 | spare tire         | 24 inch spare tire                                      |   22.2 |
-         * | 110 | jacket             | new water resistent white wind breaker                  |    0.5 |
-         * +-----+--------------------+---------------------------------------------------------+--------+
+         * +-----+--------------------+-------------------------------------------------
+         * --------+--------+
+         * | id | name | description | weight |
+         * +-----+--------------------+-------------------------------------------------
+         * --------+--------+
+         * | 101 | scooter | Small 2-wheel scooter | 3.14 |
+         * | 102 | car battery | 12V car battery | 8.1 |
+         * | 103 | 12-pack drill bits | 12-pack of drill bits with sizes ranging from
+         * #40 to #3 | 0.8 |
+         * | 104 | hammer | 12oz carpenter's hammer | 0.75 |
+         * | 105 | hammer | 14oz carpenter's hammer | 0.875 |
+         * | 106 | hammer | 18oz carpenter hammer | 1 |
+         * | 107 | rocks | box of assorted rocks | 5.1 |
+         * | 108 | jacket | water resistent black wind breaker | 0.1 |
+         * | 109 | spare tire | 24 inch spare tire | 22.2 |
+         * | 110 | jacket | new water resistent white wind breaker | 0.5 |
+         * +-----+--------------------+-------------------------------------------------
+         * --------+--------+
          * </pre>
          */
 
@@ -707,7 +718,7 @@ public class PostgreSQLConnectorITCase extends PostgresTestBase {
                     "spare tire,22.200"
                 };
 
-        List<String> actual = TestValuesTableFactory.getResults("sink");
+        List<String> actual = TestValuesTableFactory.getResultsAsStrings("sink");
         assertThat(actual, containsInAnyOrder(expected));
 
         result.getJobClient().get().cancel().get();
