@@ -21,15 +21,20 @@ import org.apache.flink.api.common.state.ListState;
 import org.apache.flink.api.common.state.ListStateDescriptor;
 import org.apache.flink.api.common.state.OperatorStateStore;
 import org.apache.flink.api.java.tuple.Tuple2;
+import org.apache.flink.api.java.tuple.Tuple3;
 import org.apache.flink.cdc.common.data.binary.BinaryRecordData;
 import org.apache.flink.cdc.common.event.CreateTableEvent;
 import org.apache.flink.cdc.common.event.DataChangeEvent;
+import org.apache.flink.cdc.common.event.DropTableEvent;
 import org.apache.flink.cdc.common.event.Event;
 import org.apache.flink.cdc.common.event.SchemaChangeEvent;
 import org.apache.flink.cdc.common.event.TableId;
+import org.apache.flink.cdc.common.event.TruncateTableEvent;
 import org.apache.flink.cdc.common.schema.Column;
 import org.apache.flink.cdc.common.schema.Schema;
 import org.apache.flink.cdc.common.schema.Selectors;
+import org.apache.flink.cdc.common.source.SupportedMetadataColumn;
+import org.apache.flink.cdc.common.utils.Preconditions;
 import org.apache.flink.cdc.common.utils.SchemaUtils;
 import org.apache.flink.cdc.runtime.parser.TransformParser;
 import org.apache.flink.runtime.state.StateInitializationContext;
@@ -67,10 +72,11 @@ public class PreTransformOperator extends AbstractStreamOperator<Event>
     private final Map<TableId, PreTransformChangeInfo> preTransformChangeInfoMap;
     private final List<Tuple2<Selectors, SchemaMetadataTransform>> schemaMetadataTransformers;
     private transient ListState<byte[]> state;
-    private final List<Tuple2<String, String>> udfFunctions;
+    private final List<Tuple3<String, String, Map<String, String>>> udfFunctions;
     private List<UserDefinedFunctionDescriptor> udfDescriptors;
     private Map<TableId, PreTransformProcessor> preTransformProcessorMap;
     private Map<TableId, Boolean> hasAsteriskMap;
+    private final boolean canContainDistributedTables;
 
     public static PreTransformOperator.Builder newBuilder() {
         return new PreTransformOperator.Builder();
@@ -79,12 +85,23 @@ public class PreTransformOperator extends AbstractStreamOperator<Event>
     /** Builder of {@link PreTransformOperator}. */
     public static class Builder {
         private final List<TransformRule> transformRules = new ArrayList<>();
+        private boolean canContainDistributedTables;
 
-        private final List<Tuple2<String, String>> udfFunctions = new ArrayList<>();
+        private final List<Tuple3<String, String, Map<String, String>>> udfFunctions =
+                new ArrayList<>();
 
         public PreTransformOperator.Builder addTransform(
                 String tableInclusions, @Nullable String projection, @Nullable String filter) {
-            transformRules.add(new TransformRule(tableInclusions, projection, filter, "", "", ""));
+            transformRules.add(
+                    new TransformRule(
+                            tableInclusions,
+                            projection,
+                            filter,
+                            "",
+                            "",
+                            "",
+                            null,
+                            new SupportedMetadataColumn[0]));
             return this;
         }
 
@@ -94,7 +111,9 @@ public class PreTransformOperator extends AbstractStreamOperator<Event>
                 @Nullable String filter,
                 String primaryKey,
                 String partitionKey,
-                String tableOption) {
+                String tableOption,
+                @Nullable String postTransformConverter,
+                SupportedMetadataColumn[] supportedMetadataColumns) {
             transformRules.add(
                     new TransformRule(
                             tableInclusions,
@@ -102,29 +121,42 @@ public class PreTransformOperator extends AbstractStreamOperator<Event>
                             filter,
                             primaryKey,
                             partitionKey,
-                            tableOption));
+                            tableOption,
+                            postTransformConverter,
+                            supportedMetadataColumns));
             return this;
         }
 
         public PreTransformOperator.Builder addUdfFunctions(
-                List<Tuple2<String, String>> udfFunctions) {
+                List<Tuple3<String, String, Map<String, String>>> udfFunctions) {
             this.udfFunctions.addAll(udfFunctions);
             return this;
         }
 
+        public PreTransformOperator.Builder canContainDistributedTables(
+                boolean canContainDistributedTables) {
+            this.canContainDistributedTables = canContainDistributedTables;
+            return this;
+        }
+
         public PreTransformOperator build() {
-            return new PreTransformOperator(transformRules, udfFunctions);
+            return new PreTransformOperator(
+                    transformRules, udfFunctions, canContainDistributedTables);
         }
     }
 
     private PreTransformOperator(
-            List<TransformRule> transformRules, List<Tuple2<String, String>> udfFunctions) {
-        this.transformRules = transformRules;
+            List<TransformRule> transformRules,
+            List<Tuple3<String, String, Map<String, String>>> udfFunctions,
+            boolean canContainDistributedTables) {
         this.preTransformChangeInfoMap = new ConcurrentHashMap<>();
         this.preTransformProcessorMap = new ConcurrentHashMap<>();
         this.schemaMetadataTransformers = new ArrayList<>();
         this.chainingStrategy = ChainingStrategy.ALWAYS;
+
+        this.transformRules = transformRules;
         this.udfFunctions = udfFunctions;
+        this.canContainDistributedTables = canContainDistributedTables;
     }
 
     @Override
@@ -135,7 +167,7 @@ public class PreTransformOperator extends AbstractStreamOperator<Event>
         super.setup(containingTask, config, output);
         this.udfDescriptors =
                 this.udfFunctions.stream()
-                        .map(udf -> new UserDefinedFunctionDescriptor(udf.f0, udf.f1))
+                        .map(udf -> new UserDefinedFunctionDescriptor(udf.f0, udf.f1, udf.f2))
                         .collect(Collectors.toList());
 
         // Initialize data fields in advance because they might be accessed in
@@ -167,6 +199,12 @@ public class PreTransformOperator extends AbstractStreamOperator<Event>
     @Override
     public void initializeState(StateInitializationContext context) throws Exception {
         super.initializeState(context);
+        if (canContainDistributedTables) {
+            // In distributed mode, we don't have a globally consistent schema for each partition.
+            // It's not meaningful to persist them to state. Instead, we rely on each source
+            // partition to send fresh CreateTableEvent to instantiate each event flow.
+            return;
+        }
         OperatorStateStore stateStore = context.getOperatorStateStore();
         ListStateDescriptor<byte[]> descriptor =
                 new ListStateDescriptor<>("originalSchemaState", byte[].class);
@@ -197,6 +235,10 @@ public class PreTransformOperator extends AbstractStreamOperator<Event>
     @Override
     public void snapshotState(StateSnapshotContext context) throws Exception {
         super.snapshotState(context);
+        if (canContainDistributedTables) {
+            // Same reason in this#initializeState.
+            return;
+        }
         state.update(
                 new ArrayList<>(
                         preTransformChangeInfoMap.values().stream()
@@ -232,6 +274,11 @@ public class PreTransformOperator extends AbstractStreamOperator<Event>
             CreateTableEvent createTableEvent = (CreateTableEvent) event;
             preTransformProcessorMap.remove(createTableEvent.tableId());
             output.collect(new StreamRecord<>(cacheCreateTable(createTableEvent)));
+        } else if (event instanceof DropTableEvent) {
+            preTransformProcessorMap.remove(((DropTableEvent) event).tableId());
+            output.collect(new StreamRecord<>(event));
+        } else if (event instanceof TruncateTableEvent) {
+            output.collect(new StreamRecord<>(event));
         } else if (event instanceof SchemaChangeEvent) {
             SchemaChangeEvent schemaChangeEvent = (SchemaChangeEvent) event;
             preTransformProcessorMap.remove(schemaChangeEvent.tableId());
@@ -255,7 +302,6 @@ public class PreTransformOperator extends AbstractStreamOperator<Event>
     private Optional<SchemaChangeEvent> cacheChangeSchema(SchemaChangeEvent event) {
         TableId tableId = event.tableId();
         PreTransformChangeInfo tableChangeInfo = preTransformChangeInfoMap.get(tableId);
-
         Schema originalSchema =
                 SchemaUtils.applySchemaChangeEvent(tableChangeInfo.getSourceSchema(), event);
         Schema preTransformedSchema = tableChangeInfo.getPreTransformedSchema();
@@ -343,17 +389,8 @@ public class PreTransformOperator extends AbstractStreamOperator<Event>
             if (!transform.getSelectors().isMatch(tableId)) {
                 continue;
             }
-            if (!transform.getProjection().isPresent()) {
-                processProjectionTransform(tableId, tableSchema, referencedColumnsSet, null);
-                hasMatchTransform = true;
-            } else {
-                TransformProjection transformProjection = transform.getProjection().get();
-                if (transformProjection.isValid()) {
-                    processProjectionTransform(
-                            tableId, tableSchema, referencedColumnsSet, transform);
-                    hasMatchTransform = true;
-                }
-            }
+            processProjectionTransform(tableId, tableSchema, referencedColumnsSet, transform);
+            hasMatchTransform = true;
         }
         if (!hasMatchTransform) {
             processProjectionTransform(tableId, tableSchema, referencedColumnsSet, null);
@@ -420,20 +457,25 @@ public class PreTransformOperator extends AbstractStreamOperator<Event>
 
     private DataChangeEvent processDataChangeEvent(DataChangeEvent dataChangeEvent) {
         if (!transforms.isEmpty()) {
-            PreTransformProcessor preTransformProcessor =
-                    preTransformProcessorMap.get(dataChangeEvent.tableId());
+            TableId tableId = dataChangeEvent.tableId();
+            PreTransformProcessor processor = preTransformProcessorMap.get(tableId);
+            Preconditions.checkArgument(
+                    processor != null,
+                    "Transform operator receives a data change event from table %s without a full schema view. "
+                            + "This might happen if source with distributed tables doesn't emit CreateTableEvent first after fail-over. "
+                            + "This is likely a bug, please consider filing an issue.",
+                    tableId);
+
             BinaryRecordData before = (BinaryRecordData) dataChangeEvent.before();
             BinaryRecordData after = (BinaryRecordData) dataChangeEvent.after();
             if (before != null) {
-                BinaryRecordData projectedBefore =
-                        preTransformProcessor.processFillDataField(before);
+                BinaryRecordData projectedBefore = processor.processFillDataField(before);
                 dataChangeEvent = DataChangeEvent.projectBefore(dataChangeEvent, projectedBefore);
             }
             if (after != null) {
-                BinaryRecordData projectedAfter = preTransformProcessor.processFillDataField(after);
+                BinaryRecordData projectedAfter = processor.processFillDataField(after);
                 dataChangeEvent = DataChangeEvent.projectAfter(dataChangeEvent, projectedAfter);
             }
-            return dataChangeEvent;
         }
         return dataChangeEvent;
     }
