@@ -19,7 +19,9 @@ package org.apache.flink.cdc.runtime.parser;
 
 import org.apache.flink.api.common.io.ParseException;
 import org.apache.flink.cdc.common.schema.Column;
+import org.apache.flink.cdc.common.source.SupportedMetadataColumn;
 import org.apache.flink.cdc.common.types.DataType;
+import org.apache.flink.cdc.common.utils.Preconditions;
 import org.apache.flink.cdc.runtime.operators.transform.ProjectionColumn;
 import org.apache.flink.cdc.runtime.operators.transform.UserDefinedFunctionDescriptor;
 import org.apache.flink.cdc.runtime.parser.metadata.TransformSchemaFactory;
@@ -36,6 +38,7 @@ import org.apache.calcite.prepare.CalciteCatalogReader;
 import org.apache.calcite.rel.RelNode;
 import org.apache.calcite.rel.RelRoot;
 import org.apache.calcite.rel.type.RelDataType;
+import org.apache.calcite.rel.type.RelDataTypeFactory;
 import org.apache.calcite.rel.type.RelDataTypeField;
 import org.apache.calcite.rel.type.RelDataTypeSystem;
 import org.apache.calcite.rex.RexBuilder;
@@ -71,16 +74,17 @@ import org.slf4j.LoggerFactory;
 import javax.annotation.Nullable;
 
 import java.util.ArrayList;
-import java.util.Arrays;
+import java.util.Collections;
 import java.util.HashMap;
 import java.util.HashSet;
 import java.util.List;
 import java.util.Map;
-import java.util.Optional;
+import java.util.Objects;
 import java.util.Properties;
 import java.util.Set;
 import java.util.stream.Collectors;
 import java.util.stream.IntStream;
+import java.util.stream.Stream;
 
 import static org.apache.flink.cdc.common.utils.StringUtils.isNullOrWhitespaceOnly;
 import static org.apache.flink.cdc.runtime.parser.metadata.MetadataColumns.METADATA_COLUMNS;
@@ -104,8 +108,10 @@ public class TransformParser {
     private static RelNode sqlToRel(
             List<Column> columns,
             SqlNode sqlNode,
-            List<UserDefinedFunctionDescriptor> udfDescriptors) {
-        List<Column> columnsWithMetadata = copyFillMetadataColumn(columns);
+            List<UserDefinedFunctionDescriptor> udfDescriptors,
+            SupportedMetadataColumn[] supportedMetadataColumns) {
+        List<Column> columnsWithMetadata =
+                copyFillMetadataColumn(columns, supportedMetadataColumns);
         CalciteSchema rootSchema = CalciteSchema.createRootSchema(true);
         SchemaPlus schema = rootSchema.plus();
         Map<String, Object> operand = new HashMap<>();
@@ -123,10 +129,11 @@ public class TransformParser {
                 if (udf.getReturnTypeHint() != null) {
                     // This UDF has return type hint annotation
                     returnTypeInference =
-                            o ->
-                                    o.getTypeFactory()
-                                            .createSqlType(
-                                                    convertCalciteType(udf.getReturnTypeHint()));
+                            o -> {
+                                RelDataTypeFactory typeFactory = o.getTypeFactory();
+                                DataType returnTypeHint = udf.getReturnTypeHint();
+                                return convertCalciteType(typeFactory, returnTypeHint);
+                            };
                 } else {
                     // Infer it from eval method return type
                     returnTypeInference = o -> function.getReturnType(o.getTypeFactory());
@@ -158,7 +165,9 @@ public class TransformParser {
                         SqlOperatorTables.chain(transformSqlOperatorTable, udfOperatorTable),
                         calciteCatalogReader,
                         factory,
-                        SqlValidator.Config.DEFAULT.withIdentifierExpansion(true));
+                        SqlValidator.Config.DEFAULT
+                                .withIdentifierExpansion(true)
+                                .withConformance(SqlConformanceEnum.MYSQL_5));
         SqlNode validateSqlNode = validator.validate(sqlNode);
         SqlToRelConverter sqlToRelConverter =
                 new SqlToRelConverter(
@@ -169,8 +178,8 @@ public class TransformParser {
                                 new HepPlanner(new HepProgramBuilder().build()),
                                 new RexBuilder(factory)),
                         StandardConvertletTable.INSTANCE,
-                        SqlToRelConverter.config().withTrimUnusedFields(false));
-        RelRoot relRoot = sqlToRelConverter.convertQuery(validateSqlNode, false, true);
+                        SqlToRelConverter.config().withTrimUnusedFields(true));
+        RelRoot relRoot = sqlToRelConverter.convertQuery(validateSqlNode, false, false);
         return relRoot.rel;
     }
 
@@ -262,7 +271,8 @@ public class TransformParser {
     public static List<ProjectionColumn> generateProjectionColumns(
             String projectionExpression,
             List<Column> columns,
-            List<UserDefinedFunctionDescriptor> udfDescriptors) {
+            List<UserDefinedFunctionDescriptor> udfDescriptors,
+            SupportedMetadataColumn[] supportedMetadataColumns) {
         if (isNullOrWhitespaceOnly(projectionExpression)) {
             return new ArrayList<>();
         }
@@ -270,119 +280,136 @@ public class TransformParser {
         if (sqlSelect.getSelectList().isEmpty()) {
             return new ArrayList<>();
         }
+
         expandWildcard(sqlSelect, columns);
-        RelNode relNode = sqlToRel(columns, sqlSelect, udfDescriptors);
+        RelNode relNode = sqlToRel(columns, sqlSelect, udfDescriptors, supportedMetadataColumns);
         Map<String, RelDataType> relDataTypeMap =
                 relNode.getRowType().getFieldList().stream()
                         .collect(
                                 Collectors.toMap(
                                         RelDataTypeField::getName, RelDataTypeField::getType));
-
-        Map<String, DataType> rawDataTypeMap =
-                columns.stream().collect(Collectors.toMap(Column::getName, Column::getType));
-
-        Map<String, Boolean> isNotNullMap =
-                columns.stream()
-                        .collect(
-                                Collectors.toMap(
-                                        Column::getName, column -> !column.getType().isNullable()));
-
+        Map<String, Column> originalColumnMap =
+                columns.stream().collect(Collectors.toMap(Column::getName, column -> column));
         List<ProjectionColumn> projectionColumns = new ArrayList<>();
+        Map<String, Integer> addedProjectionColumnNames = new HashMap<>();
 
         for (SqlNode sqlNode : sqlSelect.getSelectList()) {
+            ProjectionColumn projectionColumn;
+
+            // A projection column could be <EXPR> AS <IDENTIFIER>...
             if (sqlNode instanceof SqlBasicCall) {
                 SqlBasicCall sqlBasicCall = (SqlBasicCall) sqlNode;
-                if (SqlKind.AS.equals(sqlBasicCall.getOperator().kind)) {
-                    Optional<SqlNode> transformOptional = Optional.empty();
-                    String columnName;
-                    List<SqlNode> operandList = sqlBasicCall.getOperandList();
-                    if (operandList.size() == 2) {
-                        transformOptional = Optional.of(operandList.get(0));
-                        SqlNode sqlNode1 = operandList.get(1);
-                        if (sqlNode1 instanceof SqlIdentifier) {
-                            SqlIdentifier sqlIdentifier = (SqlIdentifier) sqlNode1;
-                            columnName = sqlIdentifier.names.get(sqlIdentifier.names.size() - 1);
-                        } else {
-                            columnName = null;
-                        }
-                    } else {
-                        columnName = null;
-                    }
-                    if (isMetadataColumn(columnName)) {
-                        continue;
-                    }
-                    ProjectionColumn projectionColumn =
-                            transformOptional.isPresent()
-                                    ? ProjectionColumn.of(
-                                            columnName,
-                                            DataTypeConverter.convertCalciteRelDataTypeToDataType(
-                                                    relDataTypeMap.get(columnName)),
-                                            transformOptional.get().toString(),
-                                            JaninoCompiler.translateSqlNodeToJaninoExpression(
-                                                    transformOptional.get(), udfDescriptors),
-                                            parseColumnNameList(transformOptional.get()))
-                                    : ProjectionColumn.of(
-                                            columnName,
-                                            DataTypeConverter.convertCalciteRelDataTypeToDataType(
-                                                    relDataTypeMap.get(columnName)));
-                    boolean hasReplacedDuplicateColumn = false;
-                    for (int i = 0; i < projectionColumns.size(); i++) {
-                        if (projectionColumns.get(i).getColumnName().equals(columnName)
-                                && !projectionColumns.get(i).isValidTransformedProjectionColumn()) {
-                            hasReplacedDuplicateColumn = true;
-                            projectionColumns.set(i, projectionColumn);
-                            break;
-                        }
-                    }
-                    if (!hasReplacedDuplicateColumn) {
-                        projectionColumns.add(projectionColumn);
-                    }
+                List<SqlNode> operandList = sqlBasicCall.getOperandList();
+                Preconditions.checkArgument(
+                        SqlKind.AS.equals(sqlBasicCall.getOperator().kind)
+                                && operandList.size() == 2
+                                && operandList.get(1) instanceof SqlIdentifier,
+                        "Unrecognized projection expression: "
+                                + sqlBasicCall
+                                + ". Should be <EXPR> AS <IDENTIFIER>");
+
+                // It's the identifier node for aliased column.
+                SqlIdentifier aliasNode = (SqlIdentifier) operandList.get(1);
+                String columnName = aliasNode.names.get(aliasNode.names.size() - 1);
+
+                Preconditions.checkArgument(
+                        !isMetadataColumn(columnName, supportedMetadataColumns),
+                        "Column name %s is reserved and shading it is not allowed.",
+                        columnName);
+
+                // This is the actual expression node of this projection column.
+                SqlNode exprNode = operandList.get(0);
+
+                if (exprNode instanceof SqlIdentifier) {
+                    // This is a simple column rename like col_a AS col_b. Simply forward it to
+                    // avoid losing metadata info like comments and default expressions.
+                    SqlIdentifier identifierExprNode = (SqlIdentifier) exprNode;
+                    String originalName =
+                            identifierExprNode.names.get(identifierExprNode.names.size() - 1);
+                    projectionColumn =
+                            resolveProjectionColumnFromIdentifier(
+                                    relDataTypeMap,
+                                    originalColumnMap,
+                                    originalName,
+                                    columnName,
+                                    supportedMetadataColumns);
                 } else {
-                    throw new ParseException(
-                            "Unrecognized projection expression: "
-                                    + sqlBasicCall
-                                    + ". Should be <EXPR> AS <IDENTIFIER>");
+                    projectionColumn =
+                            ProjectionColumn.ofCalculated(
+                                    columnName,
+                                    DataTypeConverter.convertCalciteRelDataTypeToDataType(
+                                            relDataTypeMap.get(columnName)),
+                                    exprNode.toString(),
+                                    JaninoCompiler.translateSqlNodeToJaninoExpression(
+                                            exprNode, udfDescriptors),
+                                    parseColumnNameList(exprNode));
                 }
-            } else if (sqlNode instanceof SqlIdentifier) {
+            }
+            // ... or an existing column's name identifier.
+            else if (sqlNode instanceof SqlIdentifier) {
                 SqlIdentifier sqlIdentifier = (SqlIdentifier) sqlNode;
                 String columnName = sqlIdentifier.names.get(sqlIdentifier.names.size() - 1);
-                DataType columnType;
-                if (rawDataTypeMap.containsKey(columnName)) {
-                    columnType = rawDataTypeMap.get(columnName);
-                } else if (relDataTypeMap.containsKey(columnName)) {
-                    columnType =
-                            DataTypeConverter.convertCalciteRelDataTypeToDataType(
-                                    relDataTypeMap.get(columnName));
-                } else {
-                    throw new RuntimeException(
-                            String.format("Failed to deduce column %s type", columnName));
-                }
-                if (isMetadataColumn(columnName)) {
-                    projectionColumns.add(
-                            ProjectionColumn.of(
-                                    columnName,
-                                    // Metadata columns should never be null
-                                    columnType.notNull(),
-                                    columnName,
-                                    columnName,
-                                    Arrays.asList(columnName)));
-                } else {
-                    // Calcite translated column type doesn't keep nullability.
-                    // Appending it manually to circumvent this problem.
-                    projectionColumns.add(
-                            ProjectionColumn.of(
-                                    columnName,
-                                    isNotNullMap.get(columnName)
-                                            ? columnType.notNull()
-                                            : columnType.nullable()));
-                }
+                projectionColumn =
+                        resolveProjectionColumnFromIdentifier(
+                                relDataTypeMap,
+                                originalColumnMap,
+                                columnName,
+                                columnName,
+                                supportedMetadataColumns);
             } else {
                 throw new ParseException("Unrecognized projection: " + sqlNode.toString());
             }
+            // Projection columns comes later could override previous ones.
+            String projectionColumnName = projectionColumn.getColumnName();
+            if (addedProjectionColumnNames.containsKey(projectionColumnName)) {
+                // If we already have one column with identical name, replace it
+                projectionColumns.set(
+                        addedProjectionColumnNames.get(projectionColumnName), projectionColumn);
+            } else {
+                // Otherwise, append it at the end. Don't forget to set the index!
+                projectionColumns.add(projectionColumn);
+                addedProjectionColumnNames.put(projectionColumnName, projectionColumns.size() - 1);
+            }
         }
         return projectionColumns;
     }
 
+    /**
+     * Create a projection column from a simple identifier node (could be an upstream physical
+     * column or a metadata column).
+     */
+    public static ProjectionColumn resolveProjectionColumnFromIdentifier(
+            Map<String, RelDataType> relDataTypeMap,
+            Map<String, Column> originalColumnMap,
+            String identifier,
+            String projectedColumnName,
+            SupportedMetadataColumn[] supportedMetadataColumns) {
+        if (isMetadataColumn(identifier, supportedMetadataColumns)) {
+            // For a metadata column, we simply generate a projection column with the same
+            return ProjectionColumn.ofCalculated(
+                    projectedColumnName,
+                    // Metadata columns should never be null
+                    DataTypeConverter.convertCalciteRelDataTypeToDataType(
+                                    relDataTypeMap.get(projectedColumnName))
+                            .notNull(),
+                    identifier,
+                    identifier,
+                    Collections.singletonList(identifier));
+        }
+
+        Preconditions.checkArgument(
+                originalColumnMap.containsKey(identifier),
+                "Referenced column %s is not present in original table.",
+                identifier);
+
+        Column column = originalColumnMap.get(identifier);
+        if (Objects.equals(identifier, projectedColumnName)) {
+            return ProjectionColumn.ofForwarded(column);
+        } else {
+            return ProjectionColumn.ofAliased(column, projectedColumnName);
+        }
+    }
+
     public static String translateFilterExpressionToJaninoExpression(
             String filterExpression, List<UserDefinedFunctionDescriptor> udfDescriptors) {
         if (isNullOrWhitespaceOnly(filterExpression)) {
@@ -396,7 +423,8 @@ public class TransformParser {
         return JaninoCompiler.translateSqlNodeToJaninoExpression(where, udfDescriptors);
     }
 
-    public static List<String> parseComputedColumnNames(String projection) {
+    public static List<String> parseComputedColumnNames(
+            String projection, SupportedMetadataColumn[] supportedMetadataColumns) {
         List<String> columnNames = new ArrayList<>();
         if (isNullOrWhitespaceOnly(projection)) {
             return columnNames;
@@ -426,10 +454,9 @@ public class TransformParser {
                 }
             } else if (sqlNode instanceof SqlIdentifier) {
                 String columnName = sqlNode.toString();
-                if (isMetadataColumn(columnName) && !columnNames.contains(columnName)) {
+                if (isMetadataColumn(columnName, supportedMetadataColumns)
+                        && !columnNames.contains(columnName)) {
                     columnNames.add(columnName);
-                } else {
-                    continue;
                 }
             } else {
                 throw new ParseException("Unrecognized projection: " + sqlNode.toString());
@@ -491,17 +518,24 @@ public class TransformParser {
         return parseSelect(statement.toString());
     }
 
-    private static List<Column> copyFillMetadataColumn(List<Column> columns) {
+    private static List<Column> copyFillMetadataColumn(
+            List<Column> columns, SupportedMetadataColumn[] supportedMetadataColumns) {
         // Add metaColumn for SQLValidator.validate
         List<Column> columnsWithMetadata = new ArrayList<>(columns);
         METADATA_COLUMNS.stream()
                 .map(col -> Column.physicalColumn(col.f0, col.f1))
                 .forEach(columnsWithMetadata::add);
+        Stream.of(supportedMetadataColumns)
+                .map(sCol -> Column.physicalColumn(sCol.getName(), sCol.getType()))
+                .forEach(columnsWithMetadata::add);
         return columnsWithMetadata;
     }
 
-    private static boolean isMetadataColumn(String columnName) {
-        return METADATA_COLUMNS.stream().anyMatch(col -> col.f0.equals(columnName));
+    private static boolean isMetadataColumn(
+            String columnName, SupportedMetadataColumn[] supportedMetadataColumns) {
+        return METADATA_COLUMNS.stream().anyMatch(col -> col.f0.equals(columnName))
+                || Stream.of(supportedMetadataColumns)
+                        .anyMatch(col -> col.getName().equals(columnName));
     }
 
     public static SqlSelect parseFilterExpression(String filterExpression) {
