@@ -25,9 +25,10 @@ import org.apache.flink.cdc.common.event.Event;
 import org.apache.flink.cdc.common.event.TableId;
 import org.apache.flink.cdc.common.schema.Column;
 import org.apache.flink.cdc.common.schema.Schema;
+import org.apache.flink.cdc.common.source.SupportedMetadataColumn;
 import org.apache.flink.cdc.common.types.DataTypes;
 import org.apache.flink.cdc.common.types.RowType;
-import org.apache.flink.cdc.runtime.testutils.operators.EventOperatorTestHarness;
+import org.apache.flink.cdc.runtime.testutils.operators.RegularEventOperatorTestHarness;
 import org.apache.flink.cdc.runtime.typeutils.BinaryRecordDataGenerator;
 import org.apache.flink.streaming.runtime.streamrecord.StreamRecord;
 
@@ -191,11 +192,13 @@ public class PreTransformOperatorTest {
                                 null,
                                 "col2",
                                 "col12",
-                                "key1=value1,key2=value2")
+                                "key1=value1,key2=value2",
+                                null,
+                                new SupportedMetadataColumn[0])
                         .build();
-        EventOperatorTestHarness<PreTransformOperator, Event>
+        RegularEventOperatorTestHarness<PreTransformOperator, Event>
                 transformFunctionEventEventOperatorTestHarness =
-                        new EventOperatorTestHarness<>(transform, 1);
+                        RegularEventOperatorTestHarness.with(transform, 1);
         // Initialization
         transformFunctionEventEventOperatorTestHarness.open();
         // Create table
@@ -289,6 +292,7 @@ public class PreTransformOperatorTest {
         Assertions.assertThat(
                         transformFunctionEventEventOperatorTestHarness.getOutputRecords().poll())
                 .isEqualTo(new StreamRecord<>(updateEventExpect));
+        transformFunctionEventEventOperatorTestHarness.close();
     }
 
     @Test
@@ -301,11 +305,13 @@ public class PreTransformOperatorTest {
                                 null,
                                 "id",
                                 "id",
-                                "key1=value1,key2=value2")
+                                "key1=value1,key2=value2",
+                                null,
+                                new SupportedMetadataColumn[0])
                         .build();
-        EventOperatorTestHarness<PreTransformOperator, Event>
+        RegularEventOperatorTestHarness<PreTransformOperator, Event>
                 transformFunctionEventEventOperatorTestHarness =
-                        new EventOperatorTestHarness<>(transform, 1);
+                        RegularEventOperatorTestHarness.with(transform, 1);
         // Initialization
         transformFunctionEventEventOperatorTestHarness.open();
         // Create table
@@ -319,6 +325,7 @@ public class PreTransformOperatorTest {
                         new StreamRecord<>(
                                 new CreateTableEvent(
                                         CUSTOMERS_TABLEID, EXPECTED_NULLABILITY_SCHEMA)));
+        transformFunctionEventEventOperatorTestHarness.close();
     }
 
     @Test
@@ -331,11 +338,13 @@ public class PreTransformOperatorTest {
                                 "newage > 17 and ref2 > 17",
                                 "id",
                                 "id",
-                                "key1=value1,key2=value2")
+                                "key1=value1,key2=value2",
+                                null,
+                                new SupportedMetadataColumn[0])
                         .build();
-        EventOperatorTestHarness<PreTransformOperator, Event>
+        RegularEventOperatorTestHarness<PreTransformOperator, Event>
                 transformFunctionEventEventOperatorTestHarness =
-                        new EventOperatorTestHarness<>(transform, 1);
+                        RegularEventOperatorTestHarness.with(transform, 1);
         // Initialization
         transformFunctionEventEventOperatorTestHarness.open();
         // Create table
@@ -426,6 +435,7 @@ public class PreTransformOperatorTest {
         Assertions.assertThat(
                         transformFunctionEventEventOperatorTestHarness.getOutputRecords().poll())
                 .isEqualTo(new StreamRecord<>(updateEventExpect));
+        transformFunctionEventEventOperatorTestHarness.close();
     }
 
     @Test
@@ -438,11 +448,13 @@ public class PreTransformOperatorTest {
                                 "newage > 17",
                                 "id",
                                 "id",
-                                "key1=value1,key2=value2")
+                                "key1=value1,key2=value2",
+                                null,
+                                new SupportedMetadataColumn[0])
                         .build();
-        EventOperatorTestHarness<PreTransformOperator, Event>
+        RegularEventOperatorTestHarness<PreTransformOperator, Event>
                 transformFunctionEventEventOperatorTestHarness =
-                        new EventOperatorTestHarness<>(transform, 1);
+                        RegularEventOperatorTestHarness.with(transform, 1);
         // Initialization
         transformFunctionEventEventOperatorTestHarness.open();
         // Create table
@@ -516,6 +528,7 @@ public class PreTransformOperatorTest {
         Assertions.assertThat(
                         transformFunctionEventEventOperatorTestHarness.getOutputRecords().poll())
                 .isEqualTo(new StreamRecord<>(updateEventExpect));
+        transformFunctionEventEventOperatorTestHarness.close();
     }
 
     @Test
@@ -528,9 +541,9 @@ public class PreTransformOperatorTest {
                                 " __table_name__ = 'metadata_table' ")
                         .build();
 
-        EventOperatorTestHarness<PreTransformOperator, Event>
+        RegularEventOperatorTestHarness<PreTransformOperator, Event>
                 transformFunctionEventEventOperatorTestHarness =
-                        new EventOperatorTestHarness<>(transform, 1);
+                        RegularEventOperatorTestHarness.with(transform, 1);
         // Initialization
         transformFunctionEventEventOperatorTestHarness.open();
         // Create table
@@ -542,6 +555,7 @@ public class PreTransformOperatorTest {
                 .isEqualTo(
                         new StreamRecord<>(
                                 new CreateTableEvent(METADATA_TABLEID, EXPECTED_METADATA_SCHEMA)));
+        transformFunctionEventEventOperatorTestHarness.close();
     }
 
     @Test
@@ -554,18 +568,22 @@ public class PreTransformOperatorTest {
                                 "age < 18",
                                 "id",
                                 null,
-                                null)
+                                null,
+                                null,
+                                new SupportedMetadataColumn[0])
                         .addTransform(
                                 CUSTOMERS_TABLEID.identifier(),
                                 "id, name as roleName",
                                 "age >= 18",
                                 "id",
                                 null,
-                                null)
+                                null,
+                                null,
+                                new SupportedMetadataColumn[0])
                         .build();
-        EventOperatorTestHarness<PreTransformOperator, Event>
+        RegularEventOperatorTestHarness<PreTransformOperator, Event>
                 transformFunctionEventEventOperatorTestHarness =
-                        new EventOperatorTestHarness<>(transform, 1);
+                        RegularEventOperatorTestHarness.with(transform, 1);
         // Initialization
         transformFunctionEventEventOperatorTestHarness.open();
         // Create table
@@ -579,6 +597,7 @@ public class PreTransformOperatorTest {
                         new StreamRecord<>(
                                 new CreateTableEvent(
                                         CUSTOMERS_TABLEID, EXPECTED_MULTITRANSFORM_SCHEMA)));
+        transformFunctionEventEventOperatorTestHarness.close();
     }
 
     @Test
@@ -591,18 +610,22 @@ public class PreTransformOperatorTest {
                                 "age < 18",
                                 "id",
                                 null,
-                                null)
+                                null,
+                                null,
+                                new SupportedMetadataColumn[0])
                         .addTransform(
                                 CUSTOMERS_TABLEID.identifier(),
                                 "id, age, name, sex, 'Juvenile' as roleName",
                                 "age >= 18",
                                 "id",
                                 null,
-                                null)
+                                null,
+                                null,
+                                new SupportedMetadataColumn[0])
                         .build();
-        EventOperatorTestHarness<PreTransformOperator, Event>
+        RegularEventOperatorTestHarness<PreTransformOperator, Event>
                 transformFunctionEventEventOperatorTestHarness =
-                        new EventOperatorTestHarness<>(transform, 1);
+                        RegularEventOperatorTestHarness.with(transform, 1);
         // Initialization
         transformFunctionEventEventOperatorTestHarness.open();
         // Create table
@@ -615,6 +638,7 @@ public class PreTransformOperatorTest {
                 .isEqualTo(
                         new StreamRecord<>(
                                 new CreateTableEvent(CUSTOMERS_TABLEID, MULTITRANSFORM_SCHEMA)));
+        transformFunctionEventEventOperatorTestHarness.close();
     }
 
     @Test
@@ -622,18 +646,27 @@ public class PreTransformOperatorTest {
         PreTransformOperator transform =
                 PreTransformOperator.newBuilder()
                         .addTransform(
-                                CUSTOMERS_TABLEID.identifier(), null, "age < 18", "id", null, null)
+                                CUSTOMERS_TABLEID.identifier(),
+                                null,
+                                "age < 18",
+                                "id",
+                                null,
+                                null,
+                                null,
+                                new SupportedMetadataColumn[0])
                         .addTransform(
                                 CUSTOMERS_TABLEID.identifier(),
                                 "id, age, UPPER(name) as name, sex",
                                 "age >= 18",
                                 "id",
                                 null,
-                                null)
+                                null,
+                                null,
+                                new SupportedMetadataColumn[0])
                         .build();
-        EventOperatorTestHarness<PreTransformOperator, Event>
+        RegularEventOperatorTestHarness<PreTransformOperator, Event>
                 transformFunctionEventEventOperatorTestHarness =
-                        new EventOperatorTestHarness<>(transform, 1);
+                        RegularEventOperatorTestHarness.with(transform, 1);
         // Initialization
         transformFunctionEventEventOperatorTestHarness.open();
         // Create table
@@ -646,5 +679,6 @@ public class PreTransformOperatorTest {
                 .isEqualTo(
                         new StreamRecord<>(
                                 new CreateTableEvent(CUSTOMERS_TABLEID, MULTITRANSFORM_SCHEMA)));
+        transformFunctionEventEventOperatorTestHarness.close();
     }
 }
