@@ -21,21 +21,24 @@ import org.apache.flink.cdc.common.data.binary.BinaryStringData;
 import org.apache.flink.cdc.common.event.CreateTableEvent;
 import org.apache.flink.cdc.common.event.DataChangeEvent;
 import org.apache.flink.cdc.common.event.FlushEvent;
+import org.apache.flink.cdc.common.event.SchemaChangeEventType;
 import org.apache.flink.cdc.common.event.TableId;
 import org.apache.flink.cdc.common.schema.Schema;
 import org.apache.flink.cdc.common.sink.DefaultDataChangeEventHashFunctionProvider;
 import org.apache.flink.cdc.common.types.DataTypes;
 import org.apache.flink.cdc.common.types.RowType;
-import org.apache.flink.cdc.runtime.testutils.operators.EventOperatorTestHarness;
+import org.apache.flink.cdc.runtime.testutils.operators.RegularEventOperatorTestHarness;
 import org.apache.flink.cdc.runtime.testutils.schema.TestingSchemaRegistryGateway;
 import org.apache.flink.cdc.runtime.typeutils.BinaryRecordDataGenerator;
 import org.apache.flink.streaming.runtime.streamrecord.StreamRecord;
 
 import org.junit.jupiter.api.Test;
 
+import java.util.Collections;
+
 import static org.assertj.core.api.Assertions.assertThat;
 
-/** Unit test for {@link PrePartitionOperator}. */
+/** Unit test for {@link RegularPrePartitionOperator}. */
 class PrePartitionOperatorTest {
     private static final TableId CUSTOMERS =
             TableId.tableId("my_company", "my_branch", "customers");
@@ -50,54 +53,60 @@ class PrePartitionOperatorTest {
 
     @Test
     void testBroadcastingSchemaChangeEvent() throws Exception {
-        try (EventOperatorTestHarness<PrePartitionOperator, PartitioningEvent> testHarness =
-                createTestHarness()) {
+        try (RegularEventOperatorTestHarness<RegularPrePartitionOperator, PartitioningEvent>
+                testHarness = createTestHarness()) {
             // Initialization
             testHarness.open();
             testHarness.registerTableSchema(CUSTOMERS, CUSTOMERS_SCHEMA);
 
             // CreateTableEvent
-            PrePartitionOperator operator = testHarness.getOperator();
+            RegularPrePartitionOperator operator = testHarness.getOperator();
             CreateTableEvent createTableEvent = new CreateTableEvent(CUSTOMERS, CUSTOMERS_SCHEMA);
             operator.processElement(new StreamRecord<>(createTableEvent));
             assertThat(testHarness.getOutputRecords()).hasSize(DOWNSTREAM_PARALLELISM);
             for (int i = 0; i < DOWNSTREAM_PARALLELISM; i++) {
                 assertThat(testHarness.getOutputRecords().poll())
-                        .isEqualTo(new StreamRecord<>(new PartitioningEvent(createTableEvent, i)));
+                        .isEqualTo(
+                                new StreamRecord<>(
+                                        PartitioningEvent.ofRegular(createTableEvent, i)));
             }
         }
     }
 
     @Test
     void testBroadcastingFlushEvent() throws Exception {
-        try (EventOperatorTestHarness<PrePartitionOperator, PartitioningEvent> testHarness =
-                createTestHarness()) {
+        try (RegularEventOperatorTestHarness<RegularPrePartitionOperator, PartitioningEvent>
+                testHarness = createTestHarness()) {
             // Initialization
             testHarness.open();
             testHarness.registerTableSchema(CUSTOMERS, CUSTOMERS_SCHEMA);
 
             // FlushEvent
-            PrePartitionOperator operator = testHarness.getOperator();
-            FlushEvent flushEvent = new FlushEvent(CUSTOMERS);
+            RegularPrePartitionOperator operator = testHarness.getOperator();
+            FlushEvent flushEvent =
+                    new FlushEvent(
+                            0,
+                            Collections.singletonList(CUSTOMERS),
+                            SchemaChangeEventType.CREATE_TABLE);
             operator.processElement(new StreamRecord<>(flushEvent));
             assertThat(testHarness.getOutputRecords()).hasSize(DOWNSTREAM_PARALLELISM);
             for (int i = 0; i < DOWNSTREAM_PARALLELISM; i++) {
                 assertThat(testHarness.getOutputRecords().poll())
-                        .isEqualTo(new StreamRecord<>(new PartitioningEvent(flushEvent, i)));
+                        .isEqualTo(new StreamRecord<>(PartitioningEvent.ofRegular(flushEvent, i)));
             }
         }
     }
 
     @Test
     void testPartitioningDataChangeEvent() throws Exception {
-        try (EventOperatorTestHarness<PrePartitionOperator, PartitioningEvent> testHarness =
-                createTestHarness()) {
+        try (RegularEventOperatorTestHarness<RegularPrePartitionOperator, PartitioningEvent>
+                testHarness = createTestHarness()) {
             // Initialization
             testHarness.open();
             testHarness.registerTableSchema(CUSTOMERS, CUSTOMERS_SCHEMA);
 
             // DataChangeEvent
-            PrePartitionOperator operator = testHarness.getOperator();
+            RegularPrePartitionOperator operator = testHarness.getOperator();
             BinaryRecordDataGenerator recordDataGenerator =
                     new BinaryRecordDataGenerator(((RowType) CUSTOMERS_SCHEMA.toRowDataType()));
             DataChangeEvent eventA =
@@ -115,13 +124,13 @@ class PrePartitionOperatorTest {
             assertThat(testHarness.getOutputRecords().poll())
                     .isEqualTo(
                             new StreamRecord<>(
-                                    new PartitioningEvent(
+                                    PartitioningEvent.ofRegular(
                                             eventA,
                                             getPartitioningTarget(CUSTOMERS_SCHEMA, eventA))));
             assertThat(testHarness.getOutputRecords().poll())
                     .isEqualTo(
                             new StreamRecord<>(
-                                    new PartitioningEvent(
+                                    PartitioningEvent.ofRegular(
                                             eventB,
                                             getPartitioningTarget(CUSTOMERS_SCHEMA, eventB))));
         }
@@ -134,12 +143,13 @@ class PrePartitionOperatorTest {
                 % DOWNSTREAM_PARALLELISM;
     }
 
-    private EventOperatorTestHarness<PrePartitionOperator, PartitioningEvent> createTestHarness() {
-        PrePartitionOperator operator =
-                new PrePartitionOperator(
+    private RegularEventOperatorTestHarness<RegularPrePartitionOperator, PartitioningEvent>
+            createTestHarness() {
+        RegularPrePartitionOperator operator =
+                new RegularPrePartitionOperator(
                         TestingSchemaRegistryGateway.SCHEMA_OPERATOR_ID,
                         DOWNSTREAM_PARALLELISM,
                         new DefaultDataChangeEventHashFunctionProvider());
-        return new EventOperatorTestHarness<>(operator, DOWNSTREAM_PARALLELISM);
+        return RegularEventOperatorTestHarness.with(operator, DOWNSTREAM_PARALLELISM);
     }
 }
