@@ -38,6 +38,8 @@ import org.apache.commons.lang3.SerializationException;
 import org.junit.Assert;
 import org.junit.jupiter.api.Test;
 import org.junit.jupiter.api.extension.ExtendWith;
+import org.slf4j.Logger;
+import org.slf4j.LoggerFactory;
 
 import java.io.ByteArrayInputStream;
 import java.io.ByteArrayOutputStream;
@@ -65,6 +67,8 @@ import static org.junit.Assert.fail;
 @ExtendWith(TestLoggerExtension.class)
 public abstract class SerializerTestBase<T> {
 
+    private static final Logger LOG = LoggerFactory.getLogger(SerializerTestBase.class);
+
     private final DeeplyEqualsChecker checker;
 
     protected SerializerTestBase() {
@@ -123,8 +127,7 @@ public abstract class SerializerTestBase<T> {
                                 + instance.getClass());
             }
         } catch (Exception e) {
-            System.err.println(e.getMessage());
-            e.printStackTrace();
+            LOG.error("", e);
             fail("Exception in test: " + e.getMessage());
         }
     }
@@ -181,8 +184,7 @@ public abstract class SerializerTestBase<T> {
             TypeSerializer<T> serializer = getSerializer();
             assertEquals(len, serializer.getLength());
         } catch (Exception e) {
-            System.err.println(e.getMessage());
-            e.printStackTrace();
+            LOG.error("", e);
             fail("Exception in test: " + e.getMessage());
         }
     }
@@ -199,8 +201,7 @@ public abstract class SerializerTestBase<T> {
                 deepEquals("Copied element is not equal to the original element.", datum, copy);
             }
         } catch (Exception e) {
-            System.err.println(e.getMessage());
-            e.printStackTrace();
+            LOG.error("", e);
             fail("Exception in test: " + e.getMessage());
         }
     }
@@ -217,8 +218,7 @@ public abstract class SerializerTestBase<T> {
                 deepEquals("Copied element is not equal to the original element.", datum, copy);
             }
         } catch (Exception e) {
-            System.err.println(e.getMessage());
-            e.printStackTrace();
+            LOG.error("", e);
             fail("Exception in test: " + e.getMessage());
         }
     }
@@ -238,8 +238,7 @@ public abstract class SerializerTestBase<T> {
                 target = copy;
             }
         } catch (Exception e) {
-            System.err.println(e.getMessage());
-            e.printStackTrace();
+            LOG.error("", e);
             fail("Exception in test: " + e.getMessage());
         }
     }
@@ -265,8 +264,7 @@ public abstract class SerializerTestBase<T> {
                 assertTrue("Trailing data available after deserialization.", in.available() == 0);
             }
         } catch (Exception e) {
-            System.err.println(e.getMessage());
-            e.printStackTrace();
+            LOG.error("", e);
             fail("Exception in test: " + e.getMessage());
         }
     }
@@ -296,8 +294,7 @@ public abstract class SerializerTestBase<T> {
                 reuseValue = deserialized;
             }
         } catch (Exception e) {
-            System.err.println(e.getMessage());
-            e.printStackTrace();
+            LOG.error("", e);
             fail("Exception in test: " + e.getMessage());
         }
     }
@@ -326,8 +323,7 @@ public abstract class SerializerTestBase<T> {
 
             assertEquals("Wrong number of elements deserialized.", testData.length, num);
         } catch (Exception e) {
-            System.err.println(e.getMessage());
-            e.printStackTrace();
+            LOG.error("", e);
             fail("Exception in test: " + e.getMessage());
         }
     }
@@ -358,8 +354,7 @@ public abstract class SerializerTestBase<T> {
 
             assertEquals("Wrong number of elements deserialized.", testData.length, num);
         } catch (Exception e) {
-            System.err.println(e.getMessage());
-            e.printStackTrace();
+            LOG.error("", e);
             fail("Exception in test: " + e.getMessage());
         }
     }
@@ -392,8 +387,7 @@ public abstract class SerializerTestBase<T> {
                         toVerify.available() == 0);
             }
         } catch (Exception e) {
-            System.err.println(e.getMessage());
-            e.printStackTrace();
+            LOG.error("", e);
             fail("Exception in test: " + e.getMessage());
         }
     }
@@ -428,8 +422,7 @@ public abstract class SerializerTestBase<T> {
 
             assertEquals("Wrong number of elements copied.", testData.length, num);
         } catch (Exception e) {
-            System.err.println(e.getMessage());
-            e.printStackTrace();
+            LOG.error("", e);
             fail("Exception in test: " + e.getMessage());
         }
     }
@@ -449,8 +442,7 @@ public abstract class SerializerTestBase<T> {
             assertEquals(
                     "The copy of the serializer is not equal to the original one.", ser1, ser2);
         } catch (Exception e) {
-            System.err.println(e.getMessage());
-            e.printStackTrace();
+            LOG.error("", e);
             fail("Exception in test: " + e.getMessage());
         }
     }
@@ -461,8 +453,7 @@ public abstract class SerializerTestBase<T> {
         try {
             NullableSerializer.checkIfNullSupported(serializer);
         } catch (Throwable t) {
-            System.err.println(t.getMessage());
-            t.printStackTrace();
+            LOG.error("", t);
             fail("Unexpected failure of null value handling: " + t.getMessage());
         }
     }
