@@ -26,7 +26,11 @@ parallelism.default: 4
 execution.checkpointing.interval: 300
 EXTRACONF
 
-File.write("#{FLINK_HOME}/conf/flink-conf.yaml", EXTRA_CONF, mode: 'a+')
+if File.file?("#{FLINK_HOME}/conf/flink-conf.yaml")
+  File.write("#{FLINK_HOME}/conf/flink-conf.yaml", EXTRA_CONF, mode: 'a+')
+else
+  File.write("#{FLINK_HOME}/conf/config.yaml", EXTRA_CONF, mode: 'a+')
+end
 
 # MySQL connector is not provided
 `wget https://repo1.maven.org/maven2/mysql/mysql-connector-java/8.0.27/mysql-connector-java-8.0.27.jar -O #{FLINK_HOME}/lib/mysql-connector-java-8.0.27.jar`
\ No newline at end of file
